from __future__ import print_function
import sys
import argparse
import logging
import textwrap

import ptah
from ptah import scripts
from ptah.populate import Populate


def main():
    parser = argparse.ArgumentParser(description="ptah populate")
    parser.add_argument('config', metavar='config',
                        help='ini config file')
    parser.add_argument('step', metavar='step', nargs='*',
                        help='list of populate steps')
    parser.add_argument('-l', action="store_true", dest='list',
                        help='list of registered populate steps')
    parser.add_argument('-a', action="store_true", dest='all',
                        help='execute all active populate steps')
    args = parser.parse_args()

    env = scripts.bootstrap(args.config)

    populate = Populate(env['registry'])

    if args.list:
        titleWrap = textwrap.TextWrapper(
            initial_indent='* ',
            subsequent_indent='  ')

        descWrap = textwrap.TextWrapper(
            initial_indent='    ',
            subsequent_indent='    ')

        print('')

        for step in populate.list_steps(all=True):
            print(titleWrap.fill('{0}: {1} ({2})'.format(
                        step['name'], step['title'],
                        'active' if step['active'] else 'inactive')))
            if step['factory'].__doc__:
                print(descWrap.fill(step['factory'].__doc__))

            print('')
    elif args.all:
        populate.execute()
    elif args.step:
        populate.execute(args.step)
    else:
        parser.print_help()

    ptah.shutdown()



from pyramid.view import view_config
from pyramid.interfaces import IRootFactory
from pyramid.traversal import DefaultRootFactory
from pyramid.httpexceptions import HTTPForbidden

import ptah
from ptah import config

CONFIG_ID = 'ptah.manage:config'
MANAGE_ID = 'ptah.manage:module'
MANAGE_ACTIONS = 'ptah.manage:actions'


class PtahModule(object):

    #: Module name (also is used for url generation)
    name = ''

    #: Module title
    title = ''

    def __init__(self, manager, request):
        self.__parent__ = manager
        self.request = request

    def url(self):
        """ return url to this module """
        return '{0}/{1}'.format(get_manage_url(self.request), self.name)

    @property
    def __name__(self):
        return self.name

    def available(self):
        """ is module available """
        return True


def get_manage_url(request):
    url = request.application_url
    if url.endswith('/'):
        url = url[:-1]

    cfg = ptah.get_settings(ptah.CFG_ID_PTAH, request.registry)
    return '{0}/{1}'.format(url, cfg['manage'])


def module(name):
    info = config.DirectiveInfo()

    def wrapper(cls):
        discr = (MANAGE_ID, name)
        intr = config.Introspectable(MANAGE_ID, discr, cls.title, MANAGE_ID)
        intr['name'] = name
        intr['factory'] = cls
        intr['codeinfo'] = info.codeinfo

        def _complete(cfg, cls, name):
            cls.name = name
            cfg.get_cfg_storage(MANAGE_ID)[name] = cls

        info.attach(
            config.Action(
                _complete, (cls, name),
                discriminator=discr, introspectables=(intr,))
            )
        return cls

    return wrapper


class PtahAccessManager(object):
    """ Allow access to ptah manage for users with specific role and id """

    def __init__(self, registry=None):
        self.cfg = ptah.get_settings(ptah.CFG_ID_PTAH, registry)

    def __call__(self, userid, request):
        managers = self.cfg['managers']

        if userid == ptah.SUPERUSER_URI or '*' in managers:
            return True

        role = self.cfg['manager_role']
        if role:
            root = getattr(request, 'root', None)
            if root is None:
                root_factory = request.registry.queryUtility(
                    IRootFactory, default=DefaultRootFactory)
                root = root_factory(request)

            if role in ptah.get_local_roles(userid, request, root):
                return True

        principal = ptah.resolve(userid)

        if principal is not None and principal.login in managers:
            return True

        return False


def check_access(userid, request):
    manager = ptah.get_settings(ptah.CFG_ID_PTAH).get('access_manager')
    if manager is not None:
        return manager(userid, request)
    return False


def set_access_manager(manager, registry=None):
    ptah.get_settings(ptah.CFG_ID_PTAH, registry)['access_manager'] = manager


class root(object):
    __name__ = None
    __parent__ = None


class PtahManageRoute(object):
    """ ptah management route """

    __name__ = 'ptah-manage'
    __parent__ = root()

    def __init__(self, request):
        self.request = request

        userid = ptah.auth_service.get_userid()
        if not check_access(userid, request):
            raise HTTPForbidden()

        self.userid = userid
        self.cfg = ptah.get_settings(ptah.CFG_ID_PTAH)
        self.__name__ = self.cfg['manage']

        ptah.auth_service.set_effective_userid(ptah.SUPERUSER_URI)

    def __getitem__(self, key):
        if key not in self.cfg['disable_modules']:
            mod = ptah.get_cfg_storage(MANAGE_ID).get(key)

            if mod is not None:
                return mod(self, self.request)

        raise KeyError(key)


ptah.layout.register(
    '', PtahManageRoute, root=PtahManageRoute, parent='ptah-manage',
    renderer="ptah.manage:templates/ptah-layout.pt")

ptah.layout.register(
    'ptah-page', PtahManageRoute, root=PtahManageRoute, parent='ptah-manage',
    renderer="ptah.manage:templates/ptah-layout.pt")

@ptah.layout(
    'ptah-manage', PtahManageRoute, root=PtahManageRoute,
    renderer="ptah.manage:templates/ptah-manage.pt")

class LayoutManage(ptah.View):
    """ Base layout for ptah manage """

    def update(self):
        self.user = ptah.resolve(self.context.userid)
        self.manage_url = get_manage_url(self.request)

        mod = self.request.context
        while not isinstance(mod, PtahModule):
            mod = getattr(mod, '__parent__', None)
            if mod is None: # pragma: no cover
                break

        self.module = mod
        self.actions = ptah.list_uiactions(mod, self.request, MANAGE_ACTIONS)


@view_config(
    context=PtahManageRoute, wrapper = ptah.wrap_layout(),
    renderer = 'ptah.manage:templates/manage.pt')

class ManageView(ptah.View):
    """List ptah modules"""

    rst_to_html = staticmethod(ptah.rst_to_html)

    def update(self):
        context = self.context
        request = self.request
        self.cfg = ptah.get_settings(ptah.CFG_ID_PTAH, request.registry)

        mods = []
        for name, mod in ptah.get_cfg_storage(MANAGE_ID).items():
            if name in self.cfg['disable_modules']:
                continue
            mod = mod(context, request)
            if not mod.available():
                continue
            mods.append((mod.title, mod))

        self.modules = [mod for _t, mod in sorted(mods)]



""" Aplication Root """
import sqlalchemy as sqla
from zope.interface import implementer

import ptah
from ptah.cms.node import Node, set_policy
from ptah.cms.container import Container
from ptah.cms.interfaces import IApplicationRoot, IApplicationPolicy


APPFACTORY_ID = 'ptah.cms:appfactory'

def get_app_factories():
    """ Get all registered application factories """
    return ptah.config.get_cfg_storage(APPFACTORY_ID)


@implementer(IApplicationRoot)
class BaseApplicationRoot(object):
    """ Base application root """

    __root_path__ = '/'

    def __resource_url__(self, request, info):
        return self.__root_path__


class ApplicationRoot(BaseApplicationRoot, Container):
    """ Persistent application root """


@implementer(IApplicationPolicy)
class ApplicationPolicy(object):
    """ Application policy """

    __name__ = ''
    __parent__ = None

    # default acl
    __acl__ = ptah.DEFAULT_ACL

    def __init__(self, request):
        self.request = request


class ApplicationFactory(object):
    """ Application factory """

    type = None

    def __init__(self, cls, path='', name='', title='',
                 policy = ApplicationPolicy, default_root = None,
                 parent_factory = None, config=None):
        self.id = '-'.join(part for part in path.split('/') if part)
        self.path = path if path.endswith('/') else '%s/'%path
        self.name = name
        self.title = title

        self.default_root = default_root
        if (self.path == '/') and default_root is None:
            self.default_root = True

        self.cls = cls
        self.type = cls.__type__
        self.policy = policy
        self.parent_factory = parent_factory

        discr = (APPFACTORY_ID, path)
        intr = ptah.config.Introspectable(
            APPFACTORY_ID, discr, name, APPFACTORY_ID)
        intr['id'] = self.id
        intr['factory'] = self

        if config is not None:
            ptah.config.get_cfg_storage(
                APPFACTORY_ID, registry=config.registry)[self.id] = self

        info = ptah.config.DirectiveInfo()
        info.attach(
            ptah.config.Action(
                lambda cfg: cfg.get_cfg_storage(APPFACTORY_ID)\
                    .update({self.id: self}),
                discriminator=discr, introspectables=(intr,))
            )

        self._sql_get_root = ptah.QueryFreezer(
            lambda: ptah.get_session().query(cls)\
                .filter(sqla.sql.and_(
                    cls.__name_id__ == sqla.sql.bindparam('name'),
                    cls.__type_id__ == sqla.sql.bindparam('type'))))

    def __call__(self, request=None):
        root = self._sql_get_root.first(name=self.name, type=self.type.__uri__)
        if root is None:
            root = self.type.create(title=self.title)
            root.__name_id__ = self.name
            root.__path__ = '/%s/'%root.__uri__
            Session = ptah.get_session()
            Session.add(root)
            Session.flush()

        root.__root_path__ = self.path
        root.__parent__ = policy = self.policy(request)
        root.__default_root__ = self.default_root

        if request is not None:
            set_policy(policy)
            request.root = root

        if self.parent_factory:
            policy.__parent__ = self.parent_factory()

        return root



from ptah.testing import TestCase


class StopExceptionTesting(TestCase):

    def test_api_stopexception_msg(self):
        from ptah import config

        err = config.StopException('Error message')

        self.assertEqual(str(err), '\nError message')
        self.assertEqual(err.print_tb(), 'Error message')

    def test_api_stopexception_exc(self):
        from ptah import config

        s_err = None
        try:
            raise ValueError('err')
        except Exception as exc:
            s_err = config.StopException(exc)

        self.assertIn("raise ValueError('err')", s_err.print_tb())


class LoadpackageTesting(TestCase):

    def test_stop_exc(self):
        from ptah import config

        err = ValueError('test')

        exc = config.StopException(err)
        self.assertIs(exc.exc, err)
        self.assertEqual(str(exc), '\ntest')



import ptah
from ptah.testing import PtahTestCase


class TestLibraryManagement(PtahTestCase):

    _init_ptah = False

    def test_library_register_fail(self):
        # path resuired
        self.assertRaises(ValueError, ptah.library, 'test')

        # resource should be registered
        self.assertRaises(ValueError, ptah.library, 'test', path='/test.js')

        # type should be 'js' or 'css'
        self.assertRaises(ValueError, ptah.library,
                          'test', path='/test.js', type='unknown')

    def test_library_simple_css(self):
        self.config.add_static_view('tests30', 'ptah.tests:static/dir1')
        ptah.library('test-lib30',path='style.css',type='css')
        self.init_ptah()

        from ptah.library import LIBRARY_ID
        lib = ptah.get_cfg_storage(LIBRARY_ID)['test-lib30']

        self.assertEqual(lib.name, 'test-lib30')
        self.assertEqual(len(lib.entries), 1)
        self.assertEqual(len(lib.entries), 1)
        self.assertTrue('style.css' in lib.entries[0].paths)

        self.assertEqual(
            repr(lib), '<ptah.Library "test-lib30">')

    def test_library_simple_js(self):
        ptah.library(
            'test-lib', path='http://ptah.org/test.js', type='js')
        self.init_ptah()
        from ptah.library import LIBRARY_ID
        lib = ptah.get_cfg_storage(LIBRARY_ID)['test-lib']

        self.assertEqual(
            lib.render(self.request),
            '<script src="http://ptah.org/test.js"> </script>')

    def test_library_render_absurls(self):
        ptah.library('test-lib', path='http://ptah.org/style.css', type='css')
        self.init_ptah()
        from ptah.library import LIBRARY_ID
        lib = ptah.get_cfg_storage(LIBRARY_ID)['test-lib']

        self.assertEqual(
            lib.render(self.request),
            '<link type="text/css" rel="stylesheet" href="http://ptah.org/style.css" />')

    def test_library_render_with_prefix_postfix(self):
        ptah.library(
            'test-lib', path='http://ptah.org/style.css', type='css',
            prefix='<!--[if lt IE 7 ]>', postfix='<![endif]-->')
        self.init_ptah()
        from ptah.library import LIBRARY_ID
        lib = ptah.get_cfg_storage(LIBRARY_ID)['test-lib']

        self.assertEqual(
            lib.render(self.request),
            '<!--[if lt IE 7 ]><link type="text/css" rel="stylesheet" href="http://ptah.org/style.css" /><![endif]-->')

    def test_library_render_with_extra(self):
        ptah.library(
            'test-lib', path='http://ptah.org/test.js', type='js',
            extra={'test': "extra"})
        self.init_ptah()
        from ptah.library import LIBRARY_ID
        lib = ptah.get_cfg_storage(LIBRARY_ID)['test-lib']

        self.assertEqual(
            lib.render(self.request),
            '<script test="extra" src="http://ptah.org/test.js"> </script>')

    def test_library_include(self):
        ptah.library(
            'test-lib2', path='http://ptah.org/style.css', type='css')
        self.init_ptah()

        ptah.include(self.request, 'test-lib2')
        self.assertEqual(
            ptah.render_includes(self.request),
            '<link type="text/css" rel="stylesheet" href="http://ptah.org/style.css" />')

    def test_library_include_errors(self):
        # render not included
        self.assertEqual(ptah.render_includes(self.request), '')

        # include unknown
        ptah.include(self.request, 'test-lib-test')
        self.assertEqual(ptah.render_includes(self.request), '')

    def test_library_include_recursive(self):
        ptah.library(
            'test-lib1-r', path='http://ptah.org/style1.css', type='css')

        ptah.library(
            'test-lib2-r', path='http://ptah.org/style2.css', type='css',
            require='test-lib1-r')

        ptah.library(
            'test-lib3-r', path='http://ptah.org/style3.css', type='css',
            require=('test-lib1-r', 'test-lib2-r'))

        ptah.library(
            'test-lib4-r', path='http://ptah.org/style4.css', type='css',
            require=('test-lib1-r', 'test-lib2-r'))
        self.init_ptah()

        ptah.include(self.request, 'test-lib3-r')
        ptah.include(self.request, 'test-lib4-r')

        self.assertEqual(
            ptah.render_includes(self.request),
"""<link type="text/css" rel="stylesheet" href="http://ptah.org/style1.css" />
<link type="text/css" rel="stylesheet" href="http://ptah.org/style2.css" />
<link type="text/css" rel="stylesheet" href="http://ptah.org/style3.css" />
<link type="text/css" rel="stylesheet" href="http://ptah.org/style4.css" />""")

    def test_library_include_resource(self):
        self.config.add_static_view('tests30', 'ptah.tests:static/dir1')
        ptah.library(
            'test-lib-30',
            path='ptah.tests:static/dir1/style.css',
            type='css')
        self.init_ptah()

        ptah.include(self.request, 'test-lib-30')

        self.assertEqual(
            ptah.render_includes(self.request),
            '<link type="text/css" rel="stylesheet" href="http://example.com/tests30/style.css" />')

    def test_library_View_include(self):
        ptah.library('test-lib', path='http://ptah.org/test.js', type='js')
        self.init_ptah()

        base = ptah.View(None, self.request)
        base.include('test-lib')

        self.assertEqual(
            base.render_includes(),
            '<script src="http://ptah.org/test.js"> </script>')



# ptah manage api

from ptah.manage.manage import module
from ptah.manage.manage import get_manage_url

from ptah.manage.manage import PtahModule
from ptah.manage.manage import PtahManageRoute

from ptah.manage.manage import check_access
from ptah.manage.manage import set_access_manager
from ptah.manage.manage import PtahAccessManager

from ptah.manage.manage import MANAGE_ACTIONS

from ptah.manage.apps import MANAGE_APP_ROUTE
from ptah.manage.apps import MANAGE_APP_CATEGORY


# pyramid include
def includeme(config):
    config.add_route(
        MANAGE_APP_ROUTE, '# {0}'.format(MANAGE_APP_ROUTE),
        use_global_views=False)



""" sqlalchemy query wrapper """
import ptah
import json
import sqlalchemy as sqla


def get_columns_order(mapper):
    if mapper.inherits is not None:
        order = get_columns_order(mapper.inherits)
    else:
        order = []

    table = mapper.local_table
    for cl in table.columns:
        order.append((table.name, cl.name))

    return order


def generate_fieldset(model, fieldNames=None, namesFilter=None,
                      skipPrimaryKey=True):
    """
    :param model: subclass of sqlalchemy.ext.declarative.declarative_base
    :param fieldNames: **optional** sequence of strings to use
    :param namesFilter: **optional** callable which takes a key and list
        of fieldNames to compute if fieldName should filtered out of Fieldset
        generation.
    :param skipPrimaryKey: **default: True** Should PrimaryKey be omitted
        from fieldset generation.
    :returns: a instance of :py:class:`ptah.form.Fieldset`
    """
    mapper = model.__mapper__
    order = get_columns_order(mapper)

    columns = []
    for attr in list(mapper.class_manager.attributes):
        cl = attr.__clause_element__()
        if isinstance(cl, sqla.Column):
            if fieldNames is not None and attr.key not in fieldNames:
                continue

            if namesFilter is not None and \
                    not namesFilter(attr.key, fieldNames):
                continue

            idx = order.index((cl.table.name, cl.name))
            columns.append((idx, attr.key, cl))

    columns = [(name, cl) for i, name, cl in sorted(columns)]

    return build_sqla_fieldset(columns, skipPrimaryKey)


mapping = {
    (sqla.Unicode, sqla.UnicodeText, sqla.String, sqla.Text): 'text',
    sqla.Integer: 'int',
    sqla.Float: 'float',
    sqla.Date: 'date',
    sqla.DateTime: 'datetime',
    sqla.Boolean: 'bool',
}


def build_sqla_fieldset(columns, skipPrimaryKey=False):
    """
    Given a list of SQLAlchemy columns generate a ptah.form.Fieldset.

    :param columns: sequence of sqlachemy.schema.Column instances
    :param skipPrimaryKey: **default: False** boolean whether to include PK
      Columns in Fieldset generation.
    :returns: a instance of :py:class:`ptah.form.Fieldset`
    """
    fields = []

    for name, cl in columns:
        if cl.info.get('skip', False):
            continue

        if 'field' in cl.info:
            field = cl.info['field']
            fields.append(field)
            continue

        if cl.primary_key and skipPrimaryKey:
            continue

        typ = cl.info.get('factory')
        if typ is None:
            typ = cl.info.get('field_type')

        if typ is None:
            for cls, field_type in mapping.items():
                if isinstance(cl.type, cls):
                    typ = field_type
                    break
        if typ is None:
            continue

        kwargs = {}
        for attr in ('missing', 'title', 'description',
                     'vocabulary', 'validator'):
            if attr in cl.info:
                kwargs[attr] = cl.info[attr]

        if cl.primary_key and (typ == 'int'):
            kwargs['readonly'] = True

        if callable(typ):
            field = typ(name, **kwargs)
        else:
            field = ptah.form.FieldFactory(typ, name, **kwargs)
        fields.append(field)

    return ptah.form.Fieldset(*fields)



import sys
import ptah
from ptah.scripts import populate
from pyramid.compat import NativeIO


class TestPopulateCommand(ptah.PtahTestCase):

    def setUp(self):
        super(TestPopulateCommand, self).setUp()

        import ptah.scripts
        self.original_bootstrap = ptah.scripts.bootstrap

        def bootstrap(*args, **kw):
            return {'registry': self.registry, 'request': self.request,
                    'app': object()}

        ptah.scripts.bootstrap = bootstrap

    def tearDown(self):
        import ptah.scripts
        ptah.scripts.bootstrap = self.original_bootstrap

        super(TestPopulateCommand, self).tearDown()

    def test_populate_no_params(self):
        sys.argv[:] = ['ptah-populate', 'ptah.ini']

        stdout = sys.stdout
        out = NativeIO()
        sys.stdout = out

        populate.main()
        sys.stdout = stdout

        val = out.getvalue()

        self.assertIn(
            'usage: ptah-populate [-h] [-l] [-a] config [step [step ...]]', val)

    def test_populate_list(self):

        def step(registry):
            """ """

        self.config.ptah_populate_step(
            'custom-step', title='Custom step',
            active=False, factory=step)

        sys.argv[:] = ['ptah-populate', 'ptah.ini', '-l']

        stdout = sys.stdout
        out = NativeIO()
        sys.stdout = out

        populate.main()
        sys.stdout = stdout

        val = out.getvalue()

        self.assertIn('* custom-step: Custom step (inactive)', val)

    def test_populate_execute_step(self):
        data = [False]
        def step(registry):
            data[0] = True

        self.config.ptah_populate_step(
            'custom-step', title='Custom step',
            active=False, factory=step)

        sys.argv[:] = ['ptah-populate', 'ptah.ini', 'custom-step']

        populate.main()

        self.assertTrue(data[0])

    def test_populate_execute_all(self):
        data = [False, False]
        def step1(registry):
            data[0] = True

        def step2(registry): # pragma: no cover
            data[0] = True

        self.config.ptah_populate_step(
            'custom-step1', title='Custom step 1',
            active=True, factory=step1)

        self.config.ptah_populate_step(
            'custom-step2', title='Custom step 2',
            active=False, factory=step2)

        sys.argv[:] = ['ptah-populate', 'ptah.ini', '-a']

        populate.main()

        self.assertTrue(data[0])
        self.assertFalse(data[1])



# -*- coding: utf-8 -*-
""" snippet tests """
from pyramid.compat import text_
from zope.interface import providedBy

import ptah
from ptah import config, view
from ptah.testing import PtahTestCase


class Context(object):
    pass


class TestView(PtahTestCase):

    _init_ptah = False

    def test_snippet_app_root(self):
        view = ptah.View(Context(), self.request)
        self.assertEqual(view.application_url, 'http://example.com')

        view = ptah.View(Context(), self.request)
        self.request.application_url = 'http://example.com/'
        self.assertEqual(view.application_url, 'http://example.com')

    def test_snippet_View(self):
        def TestSnippet(request):
            return 'test snippet'

        ptah.snippet.register('test', Context, TestSnippet)
        self.init_ptah()

        view = ptah.View(Context(), self.request)
        self.assertEqual(view.snippet('test'), 'test snippet')

    def test_snippet_View_with_error(self):
        def TestSnippet(request):
            raise 'Error'

        ptah.snippet.register('test', Context, TestSnippet)
        self.init_ptah()

        view = ptah.View(Context(), self.request)
        self.assertEqual(view.snippet('test'), '')

    def test_messages_View(self):
        from ptah import view
        self.init_ptah()

        v = view.View(None, self.request)
        v.message('message')

        self.assertEqual(
            v.render_messages(),
            text_('<div class="alert-message info">\n  <a class="close" href="#">×</a>\n  <p>message</p>\n</div>\n', 'utf-8'))


class TestSnippet(PtahTestCase):

    _init_ptah = False

    def test_snippet_register(self):
        def TestSnippet(request):
            return 'test snippet'

        view.snippet.register('test', Context, TestSnippet)
        self.init_ptah()

        res = self.registry.adapters.lookup(
            (providedBy(Context()), providedBy(self.request)),
            view.ISnippet, 'test')

        self.assertEqual(res(Context(), self.request), 'test snippet')

    def test_snippet_register_render_snippet(self):
        def TestSnippet(request):
            return 'test snippet'

        view.snippet.register('test', Context, TestSnippet)
        self.init_ptah()

        res = view.render_snippet('test', Context(), self.request)
        self.assertEqual(res.strip(), 'test snippet')

        self.assertRaises(
            Exception,
            view.render_snippet, 'unknown', Context(), self.request)

    def test_snippet_register_declarative(self):
        @view.snippet('pt')
        class TestSnippet(view.View):

            def __call__(self):
                return 'test'

        self.init_ptah()

        res = self.registry.getMultiAdapter(
            (Context(), self.request), view.ISnippet, 'pt')

        self.assertEqual(res, 'test')

    def test_snippet_register_pyramid(self):
        from pyramid.config import Configurator

        class TestSnippet(view.View):
            def __call__(self):
                return 'test'

        config = Configurator(autocommit=True)
        config.include('ptah')
        config.ptah_snippet('pt', view=TestSnippet)

        res = config.registry.getMultiAdapter(
            (Context(), self.request), view.ISnippet, 'pt')
        self.assertEqual(res, 'test')

    def test_snippet_register_with_renderer(self):

        @view.snippet('test', renderer='ptah:tests/test.pt')
        class TestSnippet(view.View):
            def __call__(self):
                return None

        self.init_ptah()

        res = self.registry.getMultiAdapter(
            (Context(), self.request), view.ISnippet, 'test')

        self.assertEqual(res.strip(), '<div>My snippet</div>')

    def test_snippet_register_without_class(self):
        view.snippet.register(
            'test', Context, renderer='ptah:tests/test.pt')
        self.init_ptah()

        res = self.registry.queryMultiAdapter(
            (Context(), self.request), view.ISnippet, 'test')

        self.assertEqual(res.strip(), '<div>My snippet</div>')


class TestStatusMessages(PtahTestCase):

    _init_ptah = False

    def test_messages_addmessage(self):
        from ptah import view
        self.init_ptah()

        # add_message
        view.add_message(self.request, 'message')

        res = view.render_messages(self.request)

        self.assertEqual(
            res,
            text_('<div class="alert-message info">\n  <a class="close" href="#">×</a>\n  <p>message</p>\n</div>\n','utf-8'))

    def test_messages_warning_msg(self):
        from ptah import view
        self.init_ptah()

        view.add_message(self.request, 'warning', 'warning')
        self.assertEqual(
            view.render_messages(self.request),
            text_('<div class="alert-message warning">\n  <a class="close" href="#">×</a>\n  <p>warning</p>\n</div>\n','utf-8'))

    def test_messages_error_msg(self):
        from ptah import view
        self.init_ptah()

        view.add_message(self.request, 'error', 'error')
        self.assertEqual(
            view.render_messages(self.request),
            text_('<div class="alert-message error">\n  <a class="close" href="#">×</a>\n  <p>error</p>\n</div>\n','utf-8'))

        view.add_message(self.request, ValueError('Error'), 'error')
        self.assertEqual(
            view.render_messages(self.request),
            text_('<div class="alert-message error">\n  <a class="close" href="#">×</a>\n  <p>ValueError: Error</p>\n</div>\n','utf-8'))

    def test_messages_custom_msg(self):
        from ptah import view

        @view.snippet('custom', view.Message)
        def customMessage(context, request):
            return '<div class="customMsg">{0}</div>'.format(context.message)

        self.init_ptah()

        view.add_message(self.request, 'message', 'custom')
        self.assertEqual(
            view.render_messages(self.request),
            '<div class="customMsg">message</div>')

    def test_messages_render(self):
        from ptah import view
        self.init_ptah()

        view.add_message(self.request, 'message')
        self.assertEqual(
            view.render_messages(self.request),
            text_('<div class="alert-message info">\n  <a class="close" href="#">×</a>\n  <p>message</p>\n</div>\n','utf-8'))

        msg = view.render_messages(self.request)
        self.assertEqual(msg, '')

    def test_messages_unknown_type(self):
        from ptah import view
        self.init_ptah()

        view.add_message(self.request, 'message', 'unknown')
        self.assertEqual(
            view.render_messages(self.request),
            text_('message','utf-8'))



""" uri resolver """
import uuid
import ptah
from ptah import config

ID_RESOLVER = 'ptah:resolver'


def resolve(uri):
    """ Resolve uri, return resolved object.

    Uri contains two parts, `schema` and `uuid`. `schema` is used for
    resolver selection. `uuid` is resolver specific data. By default
    uuid is a uuid.uuid4 string.
    """
    if not uri:
        return

    try:
        schema, data = uri.split(':', 1)
    except ValueError:
        return None

    try:
        return config.get_cfg_storage(ID_RESOLVER)[schema](uri)
    except KeyError:
        pass

    return None


def extract_uri_schema(uri):
    """ Extract schema of given uri """
    if uri:
        try:
            schema, data = uri.split(':', 1)
            return schema
        except:
            pass

    return None


class resolver(object):
    """ Register resolver for given schema. `resolver` is decorator style
    registration.

        :param schema: uri schema

        Resolver interface :py:class:`ptah.interfaces.resolver`

        .. code-block:: python

          import ptah

          @ptah.resolver('custom-schema')
          def my_resolver(uri):
             ....

          # now its possible to resolver 'custom-schema:xxx' uri's
          ptah.resolve('custom-schema:xxx')
    """

    def __init__(self, schema, __depth=1):
        self.info = config.DirectiveInfo(__depth)
        self.discr = (ID_RESOLVER, schema)

        self.intr = config.Introspectable(
            ID_RESOLVER, self.discr, schema, ID_RESOLVER)
        self.intr['schema'] = schema
        self.intr['codeinfo'] = self.info.codeinfo

    def __call__(self, resolver, cfg=None):
        self.intr.title = resolver.__doc__
        self.intr['callable'] = resolver

        self.info.attach(
            config.Action(
                lambda cfg, schema, resolver:
                    cfg.get_cfg_storage(ID_RESOLVER)\
                        .update({schema: resolver}),
                (self.intr['schema'], resolver),
                discriminator=self.discr, introspectables=(self.intr,)),
            cfg)

        return resolver

    @classmethod
    def register(cls, schema, resolver):
        """ Register resolver for given schema

        :param schema: uri schema
        :param resolver: Callable object that accept one parameter.

        Example:

        .. code-block:: python

          import ptah

          def my_resolver(uri):
             ....

          ptah.resolver.register('custom-schema', my_resolver)

          # now its possible to resolver 'custom-schema:xxx' uri's
          ptah.resolve('custom-schema:xxx')

        """
        cls(schema, 2)(resolver)

    @classmethod
    def pyramid(cls, cfg, schema, resolver):
        """ pyramid configurator directive `ptah_uri_resolver`.

        .. code-block:: python

            config = Configurator()
            config.include('ptah')

            def my_resolver(uri):
                ....

            config.ptah_uri_resolver('custom-schema', my_resolver)
        """
        cls(schema, 3)(resolver, cfg)


class UriFactory(object):
    """ Uri Generator

    .. code-block:: python

      uri = UriFactory('cms-content')

      uri()
      'cms-content:f73f3266fa15438e94cca3621a3f2dbc'

    """
    def __init__(self, schema):
        self.schema = schema

    def __call__(self):
        """ Generate new uri using supplied schema """
        return '%s:%s' % (self.schema, uuid.uuid4().hex)



import ptah
from pyramid.view import render_view_to_response


class TestUriIntrospect(ptah.PtahTestCase):

    def test_uri_introspect(self):
        from ptah.manage.uri import ID_RESOLVER
        from ptah.manage.intr_renderers import UriRenderer

        def resolver(uri): # pragma: no cover
            return 'Resolved'

        self.config.ptah_uri_resolver('uri-intro-test', resolver)

        intr = self.registry.introspector.get(
            ID_RESOLVER, (ID_RESOLVER, 'uri-intro-test'))

        rendered = ptah.render_snippet(ID_RESOLVER, intr, self.request)

        self.assertIn('uri-intro-test', rendered)
        self.assertIn('ptah.manage.tests.test_intr_renderers', rendered)


class TestTypeIntrospect(ptah.PtahTestCase):

    def test_type_introspect(self):
        global Content1
        class Content1(ptah.cms.Content):
            __type__ = ptah.cms.Type('content1')

        self.config.scan(self.__class__.__module__)

        intr = self.registry.introspector.get(
            'ptah.cms:type', ('ptah.cms:type', 'content1'))

        res = ptah.render_snippet('ptah.cms:type', intr, self.request)
        self.assertIn('<small>cms-type:content1</small>', res)


class SubscribersIntrospect(ptah.PtahTestCase):

    def test_subscribers(self):
        from ptah.manage.introspect import IntrospectModule, Introspector

        mod = IntrospectModule(None, self.request)

        intr = mod['ptah.config:subscriber']

        res = render_view_to_response(intr, self.request)
        self.assertIn('Event subscribers', res.text)


class FieldIntrospect(ptah.PtahTestCase):

    def test_fields(self):
        from ptah.manage.introspect import IntrospectModule, Introspector

        mod = IntrospectModule(None, self.request)

        intr = mod['ptah.form:field']

        res = render_view_to_response(intr, self.request)
        self.assertIn('List of registered fields', res.text)



import ptah
from ptah import cms
from ptah.testing import PtahTestCase
from zope import interface
from pyramid.testing import DummyRequest
from pyramid.interfaces import IRequest, IRouteRequest
from pyramid.httpexceptions import HTTPFound
from pyramid.view import render_view_to_response


class TestAppsModule(PtahTestCase):

    _init_ptah = False

    def setUp(self):
        global TestApp1, TestApp2
        class TestApp1(cms.ApplicationRoot):
            __type__ = cms.Type('app1')
        class TestApp2(cms.ApplicationRoot):
            __type__ = cms.Type('app2')

        super(TestAppsModule, self).setUp()

    def test_apps_module(self):
        from ptah.manage.manage import PtahManageRoute
        from ptah.manage.apps import ApplicationsModule
        self.init_ptah()

        request = DummyRequest()

        ptah.auth_service.set_userid('test')
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['apps']

        self.assertIsInstance(mod, ApplicationsModule)

    def test_apps_view(self):
        from ptah.manage.apps import ApplicationsModule
        from ptah.manage.apps import ApplicationsModuleView

        cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self.init_ptah()

        request = DummyRequest()
        request.request_iface = IRequest

        mod = ApplicationsModule(None, request)

        res = render_view_to_response(mod, request, '', False)

        self.assertEqual(res.status, '200 OK')
        self.assertIn('/test1/', res.text)
        self.assertIn('/test2/', res.text)
        self.assertIn('App1', res.text)
        self.assertIn('App2', res.text)

    def test_apps_traverse(self):
        from ptah.manage.apps import ApplicationsModule

        factory1 = cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        factory2 = cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self.init_ptah()

        request = DummyRequest()

        mod = ApplicationsModule(None, request)

        item = mod['app1']
        self.assertIsInstance(item, TestApp1)
        self.assertIs(request.app_factory, factory1)

        item = mod['app2']
        self.assertIsInstance(item, TestApp2)
        self.assertIs(request.app_factory, factory2)

        self.assertRaises(KeyError, mod.__getitem__, 'app3')

    def test_apps_app_view(self):
        from ptah.manage.apps import ApplicationsModule

        factory1 = cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        self.init_ptah()

        request = DummyRequest()
        request.request_iface = self.registry.getUtility(
            IRouteRequest, name=ptah.manage.MANAGE_APP_ROUTE)
        interface.directlyProvides(request, request.request_iface)
        mod = ApplicationsModule(None, request)

        item = mod['app1']

        res = render_view_to_response(item, request)
        self.assertIn('<h1>Application: Root App 1</h1>', res.text)
        self.assertIn('<td>Root App 1</td>', res.text)


class TestAppSharingForm(PtahTestCase):

    _init_ptah = False

    def setUp(self):
        global TestApp1, TestApp2
        class TestApp1(cms.ApplicationRoot):
            __type__ = cms.Type('app1')
        class TestApp2(cms.ApplicationRoot):
            __type__ = cms.Type('app2')

        super(TestAppSharingForm, self).setUp()

    def _make_app(self, request=None):
        from ptah.manage.manage import PtahManageRoute

        class Principal(object):
            id = 'test-user'
            uri = 'test:user'
            login = 'admin@ptahproject.org'

        principal = Principal()

        @ptah.resolver('test')
        def principalResolver(uri):
            return principal

        @ptah.principal_searcher('test')
        def principalSearcher(term):
            return (principal,)

        cms.ApplicationFactory(
            TestApp1, '/test1', 'app1', 'Root App 1')

        cms.ApplicationFactory(
            TestApp2, '/test2', 'app2', 'Root App 2')

        self.TestRole = ptah.Role('test', 'Test role')

        self.init_ptah()

        def trusted(*args):
            return True

        ptah.manage.set_access_manager(trusted, self.registry)

        if request is None:
            request = DummyRequest()

        ptah.auth_service.set_userid(ptah.SUPERUSER_URI)
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['apps']
        return mod['app1']

    def test_sharingform_search(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                POST={'form.buttons.search': 'Search',
                      'term': 'search term'}))
        form.csrf = False
        res = form.update()

        self.assertIsInstance(res, HTTPFound)
        self.assertIn('apps-sharing-term', form.request.session)
        self.assertEqual(
            form.request.session['apps-sharing-term'], 'search term')

    def test_sharingform_search_error(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                POST={'form.buttons.search': 'Search'}))
        form.csrf = False
        form.update()

        self.assertIn('Please specify search term',
                      ptah.render_messages(form.request))

    def test_sharingform_update(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'}))
        form.csrf = False
        form.update()

        self.assertEqual(len(form.users), 1)
        self.assertEqual(form.users[0].uri, 'test:user')
        self.assertIs(form.local_roles, app.__local_roles__)
        self.assertEqual(len(form.roles), 1)
        self.assertIs(form.roles[0], self.TestRole)
        self.assertEqual(form.get_principal('test:user').uri, 'test:user')

    def test_sharingform_save(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'},
                POST={'form.buttons.save': 'Save',
                      'userid-test:user': 'on',
                      'user-test:user-role:test': 'on'}))
        form.csrf = False
        form.update()

        self.assertIn('test:user', app.__local_roles__)
        self.assertIn(self.TestRole.id, app.__local_roles__['test:user'])

    def test_sharingform_unset(self):
        from ptah.manage.apps import SharingForm

        app = self._make_app()
        app.__local_roles__ = {'test:user': [self.TestRole.id]}

        form = SharingForm(app, DummyRequest(
                session={'apps-sharing-term': 'email'},
                POST={'form.buttons.save': 'Save',
                      'userid-test:user': 'on'}))
        form.csrf = False
        form.update()

        self.assertNotIn('test:user', app.__local_roles__)



"""Form implementation"""
from collections import OrderedDict
from webob.multidict import MultiDict
from pyramid import security
from pyramid.decorator import reify
from pyramid.renderers import render, NullRendererHelper
from pyramid.interfaces import IResponse
from pyramid.httpexceptions import HTTPForbidden
from pyramid.config.views import DefaultViewMapper

import ptah
from ptah.view import Message
from ptah.form.field import Field, Fieldset
from ptah.form.button import Buttons, Actions
from ptah.form.interfaces import _, Invalid, FORM_INPUT, FORM_DISPLAY, IForm


@ptah.snippet('form-error', Message,
              renderer='ptah.form:templates/form-error.pt')
def formErrorMessage(context, request):
    """ form error renderer """
    errors = [err for err in context.message
              if isinstance(err, Invalid) and err.field is None]

    return {'errors': errors}


class FormWidgets(OrderedDict):
    """ Form widgets manager.
    Widget is bound to content field. """

    mode = FORM_INPUT
    prefix = 'widgets.'
    fieldsets = ()

    def __init__(self, fields, form, request):
        self.form_fields = fields
        self.form = form
        self.request = request

        super(FormWidgets, self).__init__()

    def fields(self):
        return self.fieldset.fields()

    def update(self):
        params = self.form.form_params()
        content = self.form.form_content()
        prefix = '%s%s' % (self.form.prefix, self.prefix)

        self.fieldset = self.form_fields.bind(
            content, params, self.form.context)
        self.fieldsets = fieldsets = []

        # Walk through each field, making a widget out of it.
        for fieldset in self.fieldset.fieldsets():
            widgets = []

            for widget in fieldset.fields():
                if widget.mode is None:
                    widget.mode = self.mode
                widget.id = ('%s%s' % (prefix, widget.name)).replace('.', '-')
                widget.update(self.request)
                widgets.append(widget)
                self[widget.name] = widget

            fieldsets.append(
                {'fieldset': fieldset,
                 'name': fieldset.name,
                 'title': fieldset.title,
                 'widgets': widgets})

    def extract(self):
        data, errors = self.fieldset.extract()

        # additional form validation
        self.form.validate(data, errors)

        # set errors to fields
        for err in errors:
            if isinstance(err.field, Field) and err.field.error is None:
                err.field.error = err

        return data, errors


class FormViewMapper(DefaultViewMapper):

    def __init__(self, **kw):
        super(FormViewMapper, self).__init__(**kw)

        renderer = kw.get('renderer')
        if not (renderer is None or isinstance(renderer, NullRendererHelper)):
            self.map_class_native = self.map_class_native_update

    def map_class_native_update(self, form_view):
        def _class_view(context, request, _view=form_view):
            inst = _view(context, request)
            request.__original_view__ = inst
            res = inst.render_update()
            request.__view__ = inst
            return res
        return _class_view


class Form(ptah.View):
    """ A form

    ``id``: Form id

    ``name``: Form name

    ``label``: Form label

    ``description``: Form description

    ``prefix``: Form prefix, it used for html elements `id` generations.

    ``fields``: Form fields :py:class:`ptah.form.Fieldset`

    ``buttons``: Form buttons :py:class:`ptah.form.Buttons`

    ``actions``: Instance of :py:class:`ptah.form.Actions` class

    ``widgets``: Instance of :py:class:`FormWidgets` class

    ``content``: Form content, it should be `None` or dictionary with
    data for fields.

    ``params``: None

    ``mode``: Form mode. It can be :py:data:`ptah.form.FORM_INPUT` or
        :py:data:`ptah.form.FORM_DISPLAY`

    ``action``: Form action, by default ``request.url``

    ``method``: HTML Form method (`post`, `get`)

    ``csrf``: Enable/disable form csrf protection

    ``token``: csrf token

    ``csrf-name``: csrf field name

    """

    fields = Fieldset()

    buttons = None

    label = None

    description = ''

    prefix = 'form.'

    actions = None

    widgets = None

    content = None

    mode = FORM_INPUT

    method = 'post'
    enctype = 'multipart/form-data'
    accept = None
    acceptCharset = None
    params = None
    context = None

    csrf = False

    csrfname = 'csrf-token'

    __view_mapper__ = FormViewMapper

    def __init__(self, context, request):
        super(Form, self).__init__(context, request)

        if self.buttons is None:
            self.buttons = Buttons()

    @reify
    def action(self):
        return self.request.url

    @reify
    def name(self):
        return self.prefix.strip('.')

    @reify
    def id(self):
        return self.name.replace('.', '-')

    def form_content(self):
        """ Return form content.
        By default it returns ``Form.content`` attribute. """
        return self.content

    def form_params(self):
        """ get request params """
        if self.params is not None:
            if not isinstance(self.params, MultiDict):
                return MultiDict(self.params)
            return self.params

        if self.method == 'post':
            return self.request.POST
        elif self.method == 'get':
            return self.request.GET
        else:
            return self.params

    def update_widgets(self):
        """ prepare form widgets """
        self.widgets = FormWidgets(self.fields, self, self.request)
        self.widgets.mode = self.mode
        self.widgets.update()

    def update_actions(self):
        """ Prepare form actions, this method should be called directly.
        ``Form.update`` calls this method during initialization."""
        self.actions = Actions(self, self.request)
        self.actions.update()

    @property
    def token(self):
        return self.request.session.get_csrf_token()

    def validate(self, data, errors):
        """ additional form validation """
        self.validate_csrf_token()

    def validate_csrf_token(self):
        """ csrf token validation """
        if self.csrf:
            token = self.form_params().get(self.csrfname, None)
            if token is not None:
                if self.token == token:
                    return

            raise HTTPForbidden("Form authenticator is not found.")

    def extract(self):
        """ extract form values """
        return self.widgets.extract()

    def update(self, **data):
        """ update form """
        if not self.content and data:
            self.content = data

        self.update_widgets()
        self.update_actions()

        return self.actions.execute()

    def render(self):
        """ render form """
        return self.snippet(FORM_VIEW, self)

    def render_update(self):
        result = self.update()
        if result is None:
            result = {}

        return result

    def __call__(self):
        """ update form and render form to response """
        result = self.update()

        response = self.request.registry.queryAdapterOrSelf(result, IResponse)
        if response is not None:
            return response

        response = self.request.response
        body = self.render()
        if isinstance(body, bytes):
            response.body = body # pragma: no cover
        else:
            response.text = body
        return response


class DisplayForm(Form):
    """ Special form that just display content """

    mode = FORM_DISPLAY
    params = MultiDict([])

    def form_params(self):
        return self.params


FORM_VIEW = 'form-view'
FORM_ACTIONS = 'form-actions'
FORM_WIDGET = 'form-widget'
FORM_DISPLAY_WIDGET = 'form-display-widget'

ptah.snippet.register(
    FORM_VIEW, Form,
    renderer='ptah.form:templates/form.pt')

ptah.snippet.register(
    FORM_VIEW, DisplayForm,
    renderer='ptah.form:templates/form-display.pt')

ptah.snippet.register(
    FORM_ACTIONS, Form,
    renderer='ptah.form:templates/form-actions.pt')

ptah.snippet.register(
    FORM_WIDGET, Field,
    renderer='ptah.form:templates/widget.pt')

ptah.snippet.register(
    FORM_DISPLAY_WIDGET, Field,
    renderer='ptah.form:templates/widget-display.pt')



import ptah
from ptah.testing import PtahTestCase
from pyramid.view import render_view_to_response
from pyramid.testing import DummyRequest


class TestIntrospectModule(PtahTestCase):

    _init_ptah = False

    def test_introspect_module(self):
        self.init_ptah()

        from ptah.manage.manage import PtahManageRoute
        from ptah.manage.introspect import IntrospectModule

        request = DummyRequest()

        ptah.auth_service.set_userid('test')
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ('*',)

        mr = PtahManageRoute(request)
        mod = mr['introspect']

        self.assertIsInstance(mod, IntrospectModule)

    def test_traversable(self):
        from ptah.manage.introspect import IntrospectModule, Introspector
        self.init_ptah()

        request = DummyRequest()
        mod = IntrospectModule(None, request)

        self.assertRaises(KeyError, mod.__getitem__, 'unknown')

        package = mod['ptah:resolver']
        self.assertIsInstance(package, Introspector)

    def test_view(self):
        from ptah.manage.introspect import IntrospectModule, Introspector
        self.init_ptah()

        request = DummyRequest()
        mod = IntrospectModule(None, request)

        res = render_view_to_response(mod, request)
        self.assertIn('Uri resolvers', res.text)
        self.assertIn(
          '<a href="http://example.com/ptah-manage/introspect/ptah:resolver/">',
          res.text)

    def test_intr_view(self):
        from ptah.manage.introspect import IntrospectModule, Introspector
        self.init_ptah()

        request = DummyRequest()
        mod = IntrospectModule(None, request)

        intr = mod['ptah:resolver']

        res = render_view_to_response(intr, request)
        self.assertIn('System super user', res.text)



import ptah
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.view import render_view_to_response


class TestRestInspectorModule(PtahTestCase):

    def test_rest_module(self):
        from ptah.manage.rest import RestModule
        from ptah.manage.manage import PtahManageRoute

        request = DummyRequest()

        ptah.auth_service.set_userid('test')
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ('*',)
        mr = PtahManageRoute(request)
        mod = mr['rest']

        self.assertIsInstance(mod, RestModule)

    def test_view_view(self):
        from ptah.manage.rest import RestModule, RestModuleView

        request = DummyRequest()

        mod = RestModule(None, request)

        res = render_view_to_response(mod, request, '', False)
        self.assertEqual(res.status, '200 OK')

        view = RestModuleView(mod, request)
        view.update()

        self.assertEqual(
            view.url, '{0}/__rest__/cms/'.format(request.application_url))



import os
import pkg_resources
import shutil
import subprocess
import tempfile
import time
import signal
try: # pragma: no cover
    import http as httplib
except ImportError:
    import httplib


class TemplateTest(object):

    def make_venv(self, directory): # pragma: no cover
        import virtualenv
        import sys
        from virtualenv import Logger
        logger = Logger([(Logger.level_for_integer(2), sys.stdout)])
        virtualenv.logger = logger
        virtualenv.create_environment(directory,
                                      site_packages=False,
                                      clear=False,
                                      unzip_setuptools=True,
                                      use_distribute=False)
    def install(self, tmpl_name): # pragma: no cover
        try:
            self.old_cwd = os.getcwd()
            self.directory = tempfile.mkdtemp()
            self.make_venv(self.directory)
            os.chdir(pkg_resources.get_distribution('pyramid').location)
            subprocess.check_call(
                [os.path.join(self.directory, 'bin', 'python'),
                 'setup.py', 'develop'])
            os.chdir(self.directory)
            subprocess.check_call(['bin/paster', 'create', '-t', tmpl_name,
                                   'Dingle'])
            os.chdir('Dingle')
            py = os.path.join(self.directory, 'bin', 'python')
            subprocess.check_call([py, 'setup.py', 'install'])
            subprocess.check_call([py, 'setup.py', 'test'])
            paster = os.path.join(self.directory, 'bin', 'paster')
            for ininame, hastoolbar in (('development.ini', True),
                                        ('production.ini', False)):
                proc = subprocess.Popen([paster, 'serve', ininame])
                try:
                    time.sleep(5)
                    proc.poll()
                    if proc.returncode is not None:
                        raise RuntimeError('%s didnt start' % ininame)
                    conn = httplib.HTTPConnection('localhost:6543')
                    conn.request('GET', '/')
                    resp = conn.getresponse()
                    assert resp.status == 200, ininame
                    data = resp.read()
                    toolbarchunk = '<div id="pDebug"'
                    if hastoolbar:
                        assert toolbarchunk in data, ininame
                    else:
                        assert not toolbarchunk in data, ininame
                finally:
                    if hasattr(proc, 'terminate'):
                        # 2.6+
                        proc.terminate()
                    else:
                        # 2.5
                        os.kill(proc.pid, signal.SIGTERM)
        finally:
            shutil.rmtree(self.directory)
            os.chdir(self.old_cwd)


if __name__ == '__main__':     # pragma: no cover
    if not hasattr(subprocess, 'check_call'):
        # 2.4
        def check_call(*arg, **kw):
            returncode = subprocess.call(*arg, **kw)
            if returncode:
                raise ValueError(returncode)
        subprocess.check_call = check_call

    templates = ['ptah101']

    for name in templates:
        test = TemplateTest()
        test.install(name)



""" security ptah module """
import ptah
from ptah.manage import get_manage_url
from pyramid.view import view_config


@ptah.manage.module('permissions')
class PermissionsModule(ptah.manage.PtahModule):
    __doc__ = 'A listing of all permission sets and their definitions'

    title = 'Permissions'


@view_config(
    context=PermissionsModule, wrapper=ptah.wrap_layout(),
    renderer='ptah.manage:templates/permissions.pt')
class PermissionsView(ptah.View):
    """ Permissions module default view """

    def update(self):
        self.manage = get_manage_url(self.request)
        self.permissions = sorted(ptah.get_permissions().values(),
                                  key = lambda p: p.title)

        acls = ptah.get_acls()
        self.acls = sorted([acl for acl in acls.values() if acl.id != ''],
                           key = lambda a: a.title)
        self.acls.insert(0, ptah.DEFAULT_ACL)


@view_config(
    'roles.html',
    context=PermissionsModule,
    wrapper=ptah.wrap_layout(),
    renderer='ptah.manage:templates/roles.pt')

class RolesView(ptah.View):
    """ Roles view for permissions manage module """

    def update(self):
        self.roles = sorted(ptah.get_roles().values(), key = lambda r: r.title)



import sqlalchemy
from pyramid.compat import text_
from pyramid.testing import DummyRequest
from pyramid.httpexceptions import HTTPNotFound

import ptah


class TestLayoutPreview(ptah.PtahTestCase):

    _init_ptah = False

    def test_layout_preview_notfound(self):
        from ptah.manage.layout import layoutPreview

        request = DummyRequest()

        res = layoutPreview(None,request)
        self.assertIsInstance(res, HTTPNotFound)

    def test_layout_preview(self):
        from ptah.manage.layout import layoutPreview

        class Context(object):
            """ """
            __name__ = 'test'

        def View(context, request):
            request.response.text = text_('test body')
            return request.response

        self.init_ptah()

        self.config.add_view(
            name='', context=Context, wrapper=ptah.wrap_layout(), view=View,)
        self.config.ptah_layout(
            '', parent='page', context=Context,
            renderer='ptah.manage:tests/test_layout.pt')

        request = DummyRequest()

        res = layoutPreview(Context(), request).text

        self.assertIn('"python-module": "ptah.manage.tests.test_layout"', res)
        self.assertIn('"renderer": "ptah.manage:tests/test_layout.pt"', res)



import signal
import sys
from ptah import config
from ptah.testing import TestCase


class TestShutdownHandlers(TestCase):

    def test_shutdown_handler(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            shutdownExecuted.append(True)

        shutdown = sys.modules['ptah.config']
        shutdown._shutdown = False

        err = None
        try:
            shutdown.process_shutdown(signal.SIGINT, None)
        except BaseException as e:
            err = e

        self.assertTrue(isinstance(err, KeyboardInterrupt))
        self.assertTrue(shutdownExecuted[0])

    def test_shutdown_exception_in_handler(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            raise ValueError()

        from ptah.config import shutdown
        shutdown._shutdown = False

        err = None
        try:
            shutdown.process_shutdown(signal.SIGINT, None)
        except BaseException as e:
            err = e

        self.assertFalse(isinstance(err, ValueError))

    def test_shutdown_sigterm(self):
        shutdownExecuted = []

        @config.shutdown_handler
        def shutdown():
            shutdownExecuted.append(True)

        shutdown = sys.modules['ptah.config']
        shutdown._shutdown = False
        try:
            shutdown.process_shutdown(signal.SIGTERM, None)
        except:
            pass

        self.assertTrue(shutdownExecuted[0])



""" Base content class """
import sqlalchemy as sqla
from sqlalchemy.ext.hybrid import hybrid_property
from datetime import datetime
from zope.interface import implementer
from pyramid.compat import text_type
from pyramid.threadlocal import get_current_registry

import ptah
from ptah.cms.node import Node
from ptah.cms.interfaces import Error, IContent
from ptah.cms.security import action
from ptah.cms.permissions import DeleteContent, ModifyContent


class BaseContent(Node):
    """ Base class for content objects. A content class should inherit from
    `Content` to participate in content hierarchy traversal.

    .. attribute:: __path__

       A string used by the :py:class:`ptah.cms.ContentTraverser` which is
       used for efficient resolution of URL structure to content models.
       This is internal implementation and manually editing it can break
       your hierarchy.

    .. attribute:: __name__

       This is the identifier in a container if you are using containment and
       hierarchies.

    .. attribute:: title

       Content title which is editable by end user.

    .. attribute:: description

       Content description which is editable by end user.

    .. attribute:: created

       Content creation time which is set by
       :py:func:`ptah.cms.content.createdHandler` during object creation.

       :type: :py:class:`datetime.datetime`

    .. attribute:: modified

       Content modification time which is set by
       :py:func:`ptah.cms.content.modifiedHandler` during object modification.

       :type: :py:class:`datetime.datetime`

    .. attribute:: effective

       :type: :py:class:`datetime.datetime` or None

    .. attribute:: expires

       :type: :py:class:`datetime.datetime` or None

    .. attribute:: lang

       Content language code. `en` is default value.

    """

    __tablename__ = 'ptah_content'

    __id__ = sqla.Column('id', sqla.Integer,
                         sqla.ForeignKey('ptah_nodes.id'), primary_key=True)
    __path__ = sqla.Column('path', sqla.Unicode(1024),
                           default=text_type(''), index=True)
    __name_id__ = sqla.Column('name', sqla.Unicode(255), default=text_type(''))

    title = sqla.Column(sqla.Unicode(1024), default=text_type(''))
    description = sqla.Column(sqla.UnicodeText, default=text_type(''),
                              info = {'missing': '', 'field_type': 'textarea'})

    created = sqla.Column(sqla.DateTime)
    modified = sqla.Column(sqla.DateTime)
    effective = sqla.Column(sqla.DateTime)
    expires = sqla.Column(sqla.DateTime)
    lang = sqla.Column(sqla.String(12), default='en', info={'skip':True})

    # sql queries
    _sql_get = ptah.QueryFreezer(
        lambda: ptah.get_session().query(BaseContent)
        .filter(BaseContent.__uri__ == sqla.sql.bindparam('uri')))

    _sql_get_in_parent = ptah.QueryFreezer(
        lambda: ptah.get_session().query(BaseContent)
            .filter(BaseContent.__name_id__ == sqla.sql.bindparam('key'))
            .filter(BaseContent.__parent_uri__ == sqla.sql.bindparam('parent')))

    _sql_parent = ptah.QueryFreezer(
        lambda: ptah.get_session().query(BaseContent)
            .filter(BaseContent.__uri__ == sqla.sql.bindparam('parent')))

    def __init__(self, **kw):
        super(BaseContent, self).__init__(**kw)

        if self.__name__ and self.__parent__ is not None:
            self.__path__ = '%s%s/'%(self.__parent__.__path__, self.__name__)

    @hybrid_property
    def __name__(self):
        return self.__name_id__

    @__name__.setter
    def __name__(self, value):
        self.__name_id__ = value

    def __resource_url__(self, request, info):
        return '%s%s'%(request.root.__root_path__,
                       self.__path__[len(request.root.__path__):])

    @action(permission=DeleteContent)
    def delete(self):
        parent = self.__parent__
        if parent is None:
            parent = self.__parent_ref__

        if parent is None:
            raise Error("Can't find parent")

        del parent[self]

    @action(permission=ModifyContent)
    def update(self, **data):
        if self.__type__:
            tinfo = self.__type__

            for field in tinfo.fieldset.fields():
                val = data.get(field.name, field.default)
                if val is not ptah.form.null:
                    setattr(self, field.name, val)

            get_current_registry().notify(
                ptah.events.ContentModifiedEvent(self))

    def _extra_info(self, info):
        if self.__type__:
            fieldset = self.__type__.fieldset.bind()
            for field in fieldset.fields():
                val = getattr(self, field.name, field.default)
                info[field.name] = field.serialize(val)

        info['created'] = self.created
        info['modified'] = self.modified
        info['effective'] = self.effective
        info['expires'] = self.expires

    def info(self):
        info = super(BaseContent, self).info()
        info['__name__'] = self.__name__
        info['__content__'] = True
        info['__container__'] = False
        self._extra_info(info)
        return info


@implementer(IContent)
class Content(BaseContent):
    """ Content """


@ptah.subscriber(ptah.events.ContentCreatedEvent)
def content_created_handler(ev):
    """ Assigns created, modified, __owner__
        attributes for newly created content """
    now = datetime.utcnow()
    ev.object.created = now
    ev.object.modified = now

    user = ptah.auth_service.get_userid()
    if user:
        ev.object.__owner__ = user


@ptah.subscriber(ptah.events.ContentModifiedEvent)
def content_modified_handler(ev):
    """ Updates the modified attribute on content """
    ev.object.modified = datetime.utcnow()


@ptah.subscriber(ptah.events.ContentEvent)
def content_event_handler(ev):
    """ Resend uri invalidate event """
    get_current_registry().notify(
        ptah.events.UriInvalidateEvent(ev.object.__uri__))



import sys
import logging
import signal
import traceback
from collections import defaultdict, namedtuple, OrderedDict
from pyramid.compat import text_type, string_types, NativeIO
from pyramid.registry import Introspectable
from pyramid.threadlocal import get_current_registry
from zope.interface import implementedBy
from zope.interface.interfaces import IObjectEvent

import venusian
from venusian.advice import getFrameInfo

ATTACH_ATTR = '__ptah_actions__'
ID_ADAPTER = 'ptah.config:adapter'
ID_SUBSCRIBER = 'ptah.config:subscriber'

__all__ = ('initialize', 'get_cfg_storage', 'StopException',
           'event', 'adapter', 'subscriber', 'shutdown', 'shutdown_handler',
           'Action', 'ClassAction', 'DirectiveInfo')

log = logging.getLogger('ptah')


class StopException(Exception):
    """ Special initialization exception means stop execution """

    def __init__(self, exc=None):
        self.exc = exc
        if isinstance(exc, BaseException):
            self.isexc = True
            self.exc_type, self.exc_value, self.exc_traceback = sys.exc_info()
        else:
            self.isexc = False

    def __str__(self):
        return ('\n{0}'.format(self.print_tb()))

    def print_tb(self):
        if self.isexc and self.exc_value:
            out = NativeIO()
            traceback.print_exception(
                self.exc_type, self.exc_value, self.exc_traceback, file=out)
            return out.getvalue()
        else:
            return self.exc


class ObjectEventNotify(object):

    def __init__(self, registry):
        self.registry = registry

    def __call__(self, event):
        self.registry.subscribers((event.object, event), None)


def get_cfg_storage(id, registry=None, default_factory=OrderedDict):
    """ Return current config storage """
    if registry is None:
        registry = get_current_registry()

    try:
        storage = registry.__ptah_storage__
    except AttributeError:
        storage = defaultdict(lambda: OrderedDict())
        registry.__ptah_storage__ = storage

    if id not in storage:
        storage[id] = default_factory()
    return storage[id]


def pyramid_get_cfg_storage(config, id):
    return get_cfg_storage(id, config.registry)


def adapter(*args, **kw):
    """ Register adapter """
    info = DirectiveInfo()

    required = tuple(args)
    name = kw.get('name', '')

    def wrapper(func):
        discr = (ID_ADAPTER, required, _getProvides(func), name)

        intr = Introspectable(ID_ADAPTER, discr, 'Adapter', ID_ADAPTER)
        intr['name'] = name
        intr['required'] = required
        intr['adapter'] = func
        intr['codeinfo'] = info.codeinfo

        def _register(cfg, name, func, required):
            cfg.registry.registerAdapter(func, required, name=name)

        info.attach(
            Action(
                _register, (name, func, required),
                discriminator = discr, introspectables = (intr,))
            )
        return func

    return wrapper


def subscriber(*args):
    """ Register event subscriber. """
    info = DirectiveInfo(allowed_scope=('module', 'function call'))

    def wrapper(func):
        required = tuple(args)
        discr = (ID_SUBSCRIBER, func, required)

        intr = Introspectable(ID_SUBSCRIBER, discr, 'Subscriber', ID_SUBSCRIBER)
        intr['required'] = required
        intr['handler'] = func
        intr['codeinfo'] = info.codeinfo

        def _register(cfg, func, required):
            cfg.registry.registerHandler(func, required)

        info.attach(
            Action(
                _register, (func, required),
                discriminator=discr, introspectables=(intr,))
            )
        return func

    return wrapper


def _getProvides(factory):
    p = list(implementedBy(factory))
    if len(p) == 1:
        return p[0]
    else:
        raise TypeError(
            "The adapter factory doesn't implement a single interface "
            "and no provided interface was specified.")


class Action(object):

    hash = None

    def __init__(self, callable, args=(), kw={},
                 discriminator=None, order=0, introspectables=(), info=None):
        self.callable = callable
        self.args = args
        self.kw = kw
        self.order = order
        self.info = info
        self.introspectables = introspectables
        self.discriminator = discriminator

    def __hash__(self):
        return hash(self.hash)

    def __repr__(self):
        return '<%s "%s">'%(
            self.__class__.__name__,
            self.discriminator[0] if self.discriminator else None)

    def __call__(self, cfg):
        if self.callable:
            try:
                self.callable(cfg, *self.args, **self.kw)
            except:  # pragma: no cover
                log.exception(self.discriminator)
                raise


class ClassAction(Action):

    def __call__(self, cfg):
        try:
            self.callable(cfg, self.info.context, *self.args, **self.kw)
        except:  # pragma: no cover
            log.exception(self.discriminator)
            raise


CodeInfo = namedtuple('Codeinfo', 'filename lineno function source module')


class AttachData(OrderedDict):
    """ container for Attach infos """


class DirectiveInfo(object):

    def __init__(self, depth=1, moduleLevel=False, allowed_scope=None):
        scope, module, f_locals, f_globals, codeinfo = \
            getFrameInfo(sys._getframe(depth + 1))

        if allowed_scope and scope not in allowed_scope: # pragma: no cover
            raise TypeError("This directive is not allowed "
                            "to run in this scope: %s" % scope)

        if scope == 'module':
            self.name = f_locals['__name__']
        else:
            self.name = codeinfo[2]

        self.locals = f_locals
        self.scope = scope
        self.module = module
        self.codeinfo = CodeInfo(
            codeinfo[0], codeinfo[1], codeinfo[2], codeinfo[3], module.__name__)

        if depth > 1:
            _, mod, _, _, ci = getFrameInfo(sys._getframe(2))
            self.hash = (module.__name__, codeinfo[1], mod.__name__, ci[1])
        else:
            self.hash = (module.__name__, codeinfo[1])

    @property
    def context(self):
        if self.scope == 'module': # pragma: no cover
            return self.module
        else:
            return getattr(self.module, self.name, None)

    def _runaction(self, action, cfg):
        cfg.__ptah_action__ = action
        action(cfg)

    def attach(self, action, cfg=None):
        action.info = self
        if action.hash is None:
            action.hash = self.hash

        data = getattr(self.module, ATTACH_ATTR, None)
        if data is None:
            data = AttachData()
            setattr(self.module, ATTACH_ATTR, data)

        if cfg is None and action.hash in data:
            raise TypeError(
                "Directive registered twice: %s" % (action.discriminator,))
        data[action.hash] = action

        def callback(context, name, ob):
            config = context.config.with_package(self.module)

            config.info = action.info
            config.action(
                action.discriminator, self._runaction, (action, config),
                introspectables=action.introspectables,
                order=action.order)

        venusian.attach(data, callback, category='ptah')

        if cfg is not None:
            cfg.action(
                action.discriminator,
                self._runaction, (action, cfg),
                introspectables=action.introspectables, order=action.order)

    def __repr__(self):
        filename, line, function, source, module = self.codeinfo
        return ' File "%s", line %d, in %s\n' \
               '      %s\n' % (filename, line, function, source)


handlers = []
_handler_int = signal.getsignal(signal.SIGINT)
_handler_term = signal.getsignal(signal.SIGTERM)


def shutdown_handler(handler):
    """ register shutdown handler """
    handlers.append(handler)
    return handler


def shutdown():
    """ Execute all registered shutdown handlers """
    for handler in handlers:
        try:
            handler()
        except:
            log.exception("Showndown handler: %s"%handler)
            pass


def process_shutdown(sig, frame):
    """ os signal handler """
    shutdown()

    if sig == signal.SIGINT and callable(_handler_int):
        _handler_int(sig, frame)

    if sig == signal.SIGTERM and callable(_handler_term):  # pragma: no cover
        _handler_term(sig, frame)

    if sig == signal.SIGTERM:
        raise sys.exit()


def install_sigterm_handler(): # pragma: no cover
    try:
        import mod_wsgi
    except ImportError:
        signal.signal(signal.SIGINT, process_shutdown)
        signal.signal(signal.SIGTERM, process_shutdown)



""" ptah-settings command """
import argparse
import textwrap

from collections import OrderedDict
from pyramid.compat import configparser, NativeIO

import ptah
from ptah import config, scripts
from ptah.settings import SETTINGS_OB_ID
from ptah.settings import ID_SETTINGS_GROUP


grpTitleWrap = textwrap.TextWrapper(
    initial_indent='* ',
    subsequent_indent='  ')

grpDescriptionWrap = textwrap.TextWrapper(
    initial_indent='    ',
    subsequent_indent='    ')

nameWrap = textwrap.TextWrapper(
    initial_indent='  - ',
    subsequent_indent='    ')

nameTitleWrap = textwrap.TextWrapper(
    initial_indent='       ',
    subsequent_indent='       ')

nameDescriptionWrap = textwrap.TextWrapper(
    initial_indent=' * ',
    subsequent_indent='')


def main(init=True):
    args = SettingsCommand.parser.parse_args()

    # bootstrap pyramid
    if init: # pragma: no cover
        env = scripts.bootstrap(args.config)

    cmd = SettingsCommand(args)
    cmd.run()

    ptah.shutdown()


class SettingsCommand(object):
    """ 'settings' command"""

    parser = argparse.ArgumentParser(description="ptah settings management")
    parser.add_argument('config', metavar='config',
                        help='Config file')
    parser.add_argument('-a', '--all', action="store_true",
                        dest='all',
                        help='List all registered settings')
    parser.add_argument('-l', '--list',
                        dest='section', default='',
                        help='List registered settings')
    parser.add_argument('-p', '--print', action="store_true",
                        dest='printcfg',
                        help='Print default settings in ConfigParser format')

    def __init__(self, args):
        self.config = config
        self.options = args

    def run(self):
        # print defaults
        if self.options.printcfg:
            data = config.get_cfg_storage(SETTINGS_OB_ID).export(True)

            parser = configparser.ConfigParser(dict_type=OrderedDict)
            for key, val in sorted(data.items()):
                parser.set(configparser.DEFAULTSECT,
                           key, val.replace('%', '%%'))

            fp = NativeIO()
            try:
                parser.write(fp)
            finally:
                pass

            print (fp.getvalue())
            return

        if self.options.all:
            section = ''
        else:
            section = self.options.section

        # print description
        groups = sorted(config.get_cfg_storage(ID_SETTINGS_GROUP).items(),
                        key = lambda item: item[1].__title__)

        for name, group in groups:
            if section and name != section:
                continue

            print ('')
            title = group.__title__ or name

            print (grpTitleWrap.fill('{0} ({1})'.format(title, name)))
            if group.__description__:
                print (grpDescriptionWrap.fill(
                    group.__description__))

            print ('')
            for node in group.__fields__.values():
                default = '<required>' if node.required else node.default
                print (nameWrap.fill(
                    ('%s.%s: %s (%s: %s)' % (
                        name, node.name, node.title,
                        node.__class__.__name__, default))))

                print (nameTitleWrap.fill(node.description))
                print ('')



""" resource library """
from ptah import config
from pyramid.compat import string_types

LIBRARY_ID = 'ptah.view:library'


def library(name, path='', type='',
            require='', prefix='', postfix='', extra=None):
    """
    Registers a library with one or more assets.  Used to logically group
    JS and CSS collections.  Provides ability to declarare
    dependency/requirements between library collections.

    :param name: Library collection name
    :param path: The ``path`` argument points at a file or directory on disk.
    :param type:  A string, either `css` or `js`
    :param require: A library name to be considered dependency (Optional)
    :param prefix: A string which will be generated before the library HTML
    :param postfix: A string which will be generated after the library HTML
                    An example of prefix/postfix is <!-- JS/CSS -->
    :param extra: Additional attributes for computed library tag
                  An example of extra is {'type':'text/pythonscript'}
    """
    if type not in ('js', 'css'):
        raise ValueError("Uknown type '%s'"%type)

    if isinstance(path, string_types):
        path = (path,)

    if require and isinstance(require, string_types):
        require = (require,)

    if extra is None:
        extra = {}

    discr = (LIBRARY_ID, name, tuple(path))
    intr = config.Introspectable(LIBRARY_ID, discr, name, LIBRARY_ID)
    intr['name'] = name
    intr['path'] = path
    intr['type'] = type
    intr['require'] = require
    intr['prefix'] = prefix
    intr['postfix'] = postfix
    intr['extra'] = extra

    def library_impl(cfg, name, path, type, require, prefix, postfix, extra):
        data = cfg.get_cfg_storage(LIBRARY_ID)

        lib = data.get(name)
        if lib is None:
            lib = Library(name)
            data[name] = lib

        lib.add(path, type, require, prefix, postfix, extra)

    info = config.DirectiveInfo()
    info.attach(
        config.Action(
            library_impl,
            (name, path, type, require, prefix, postfix, extra),
            discriminator=discr, introspectables=(intr,))
        )


def include(request, name):
    """
    Given a library name; the library will be attached to the request.
    See render_includes function to compute the HTML from attached libraries.

    :param request: Pyramid request
    :param name: Name of library to include
    """
    libs = getattr(request, '__includes', None)
    if libs is None:
        libs = []

    libs.append(name)
    request.__includes = libs


def render_includes(request):
    """
    Renders HTML for all included libraries for this request.

    :param request: Pyramid request
    """
    _libraries = config.get_cfg_storage(LIBRARY_ID)

    seen = set()
    libraries = []

    libs = getattr(request, '__includes', None)
    if libs is None:
        return ''

    def _process(l_id):
        lib = _libraries.get(l_id)
        if lib is None:
            seen.add(l_id)
            return

        for require in lib.require:
            if require not in seen:
                _process(require)

        seen.add(l_id)
        libraries.append(lib)

    for id in libs:
        if id not in seen:
            _process(id)

    return '\n'.join(lib.render(request) for lib in libraries)


class Entry(object):

    def __init__(self, path, type='', prefix='', postfix='', extra={}):
        self.type = type
        self.prefix = prefix
        self.postfix = postfix

        if type == 'css':
            if 'rel' not in extra:
                extra['rel'] = 'stylesheet'
            if 'type' not in extra:
                extra['type'] = 'text/css'

        self.extra = extra

        self.urls = urls = []
        self.paths = paths = []
        for p in path:
            if p.startswith('http://') or p.startswith('https://'):
                urls.append(p)
            else:
                paths.append(p)

    def render(self, request):
        urls = list(self.urls)

        for path in self.paths:
            urls.append(request.static_url(path))

        if self.type == 'css':
            s = '<link %shref="%s" />'
        else:
            s = '<script %ssrc="%s"> </script>'

        extra = ' '.join('%s="%s"'%(k,v) for k, v in self.extra.items())
        if extra:
            extra = '%s '%extra

        return '%s%s%s'%(
            self.prefix, '\n\n\t'.join(s%(extra, url) for url in urls),
            self.postfix)


class Library(object):

    def __init__(self, name):
        self.name = name
        self.require = []
        self.entries = []

    def add(self, path, type, require, prefix, postfix, extra):
        self.entries.append(
            Entry(path, type, prefix, postfix, extra))

        for req in require:
            if req not in self.require:
                self.require.append(req)

    def render(self, request):
        return '\n'.join(entry.render(request) for entry in self.entries)

    def __repr__(self):
        return '<ptah.Library "%s">'%self.name



import sys
from pyramid.router import Router
from pyramid.config import Configurator

import ptah
from ptah.testing import TestCase


class TestInitializeSql(TestCase):

    def test_ptahinit_sqla(self):
        config = Configurator(
            settings = {'sqlalchemy.url': 'sqlite://'})
        config.include('ptah')
        config.commit()

        config.ptah_init_sql()
        self.assertIsNotNone(ptah.get_base().metadata.bind)


class TestPtahInit(TestCase):

    def test_init_includeme(self):
        config = Configurator()
        config.include('ptah')
        config.commit()

        for name in ('ptah_init_settings', 'ptah_init_sql',
                     'ptah_init_manage', 'ptah_init_mailer',
                     'ptah_init_rest', 'get_cfg_storage',
                     'ptah_get_settings', 'ptah_auth_checker',
                     'ptah_auth_provider', 'ptah_principal_searcher',
                     'ptah_uri_resolver', 'ptah_password_changer',
                     'ptah_layout', 'ptah_snippet',
                     'ptah_populate', 'ptah_populate_step'):
            self.assertTrue(hasattr(config, name))

        from pyramid.interfaces import \
             IAuthenticationPolicy, IAuthorizationPolicy

        self.assertIsNotNone(
            config.registry.queryUtility(IAuthenticationPolicy))
        self.assertIsNotNone(
            config.registry.queryUtility(IAuthorizationPolicy))

    def test_init_ptah_init(self):
        config = Configurator()

        data = [False, False]

        def settings_initialized_handler(ev):
            data[1] = True

        sm = config.registry
        sm.registerHandler(
            settings_initialized_handler,
            (ptah.events.SettingsInitialized,))

        config.include('ptah')
        config.ptah_init_settings()
        config.commit()

        self.assertTrue(data[1])

    def test_init_ptah_init_settings_exception(self):
        config = Configurator(autocommit = True)

        class CustomException(Exception):
            pass

        def initialized_handler(ev):
            raise CustomException()

        sm = config.registry
        sm.registerHandler(
            initialized_handler,
            (ptah.events.SettingsInitialized,))

        config.include('ptah')

        err = None
        try:
            config.ptah_init_settings()
        except Exception as e:
            err = e

        self.assertIsInstance(err, ptah.config.StopException)
        self.assertIsInstance(err.exc, CustomException)


class TestDummyMailer(ptah.PtahTestCase):

    def test_dummy_mailer(self):
        from ptah.ptahsettings import DummyMailer

        PTAH = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)

        self.assertIsInstance(PTAH['Mailer'], DummyMailer)

        PTAH['Mailer'].send('test@example.com', 'to@example.com', 'msg')



""" ptah.form fields """
import ptah
from ptah.form.field import FIELD_ID, PREVIEW_ID
from pyramid.view import view_config


@ptah.manage.module('fields')
class FieldsModule(ptah.manage.PtahModule):
    __doc__ = ('A preview and listing of all form fields in the '
      'application. This is useful to see what fields are available. '
      'You may also interact with the field to see how it works in '
      'display mode.')

    title = 'Field types'


@view_config(
    context=FieldsModule, wrapper=ptah.wrap_layout(),
    renderer='ptah.manage:templates/fields.pt')

class FieldsView(ptah.View):
    """ Fields manage module view """

    def update(self):
        data = []

        fields = ptah.get_cfg_storage(FIELD_ID)
        previews = ptah.get_cfg_storage(PREVIEW_ID)

        for name, cls in fields.items():
            data.append({'name': name,
                         'doc': cls.__doc__,
                         'preview': previews.get(cls)})

        self.fields = sorted(data, key = lambda item: item['name'])



""" layout tests """
import os, tempfile, shutil
from zope import interface
from pyramid.compat import bytes_, text_
from pyramid.interfaces import IRequest
from pyramid.interfaces import IRouteRequest

import ptah
from ptah.testing import PtahTestCase


class TestLayout(PtahTestCase):

    _init_ptah = False

    def setUp(self):
        PtahTestCase.setUp(self)
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        PtahTestCase.tearDown(self)
        shutil.rmtree(self.dir)

    def test_layout_register_simple(self):
        from ptah.layout import query_layout
        ptah.layout.register('test')
        self.init_ptah()

        layout, context = query_layout(object(), object(), self.request, 'test')

        self.assertEqual(layout.name, 'test')
        self.assertIs(layout.original, ptah.View)

    def test_layout_register_custom_class(self):
        from ptah.layout import query_layout
        class MyLayout(ptah.View):
            pass

        ptah.layout.register('test', view=MyLayout)
        self.init_ptah()

        layout, context = query_layout(object(), object(), self.request, 'test')
        self.assertIs(layout.original, MyLayout)

    def test_layout_simple_declarative(self):
        from ptah.layout import LayoutRenderer

        @ptah.layout('test', context=Context,
                     renderer='ptah:tests/test-layout-html.pt')
        class Layout(ptah.View):
            def __call__(self):
                return None

        self.init_ptah()

        renderer = LayoutRenderer('test')
        self.request.wrapped_body = 'View: test'
        self.request.wrapped_response = self.request.response

        res = renderer(Context(), self.request).text
        self.assertEqual(res.strip(), '<html>View: test</html>')

    def test_layout_pyramid_declarative(self):
        from ptah.layout import ILayout
        from pyramid.config import Configurator

        config = Configurator(autocommit=True)
        config.include('ptah')
        self.init_ptah()

        class Layout(ptah.View):
            def __call__(self):
                """ """

        config.ptah_layout('test', view=Layout)

        layout_factory = config.registry.adapters.lookup(
            (interface.providedBy(None),
             IRequest, interface.providedBy(None)), ILayout, name='test')
        self.assertIs(layout_factory.original, Layout)

    def test_layout_simple_notfound(self):
        from ptah.layout import query_layout

        v = ptah.View(Context(Context()), self.request)
        layout, context = query_layout(object(), v.context, self.request,'test')
        self.assertTrue(layout is None)

    def test_layout_simple_chain_multi_level(self):
        from ptah.layout import query_layout, LayoutRenderer
        ptah.layout.register('test', parent='.',
                             renderer='ptah:tests/test-layout.pt')
        ptah.layout.register('', context=Root, parent=None,
                             renderer='ptah:tests/test-layout-html.pt')
        self.init_ptah()

        root = Root()
        context = Context(root)

        renderer = LayoutRenderer('test')
        self.request.wrapped_body = 'View: test'
        self.request.wrapped_response = self.request.response

        res = renderer(context, self.request).text
        self.assertTrue('<html><div>View: test</div>\n</html>' in text_(res))

    def test_layout_chain_same_layer_id_on_different_levels(self):
        from ptah.layout import LayoutRenderer
        ptah.layout.register('', context=Context, parent='.',
                             renderer='ptah:tests/test-layout.pt')
        ptah.layout.register('', context=Root, parent=None,
                             renderer='ptah:tests/test-layout-html.pt')
        self.init_ptah()

        root = Root()
        context1 = Context2(root)
        context2 = Context(context1)

        renderer = LayoutRenderer('')
        self.request.wrapped_body = 'View: test'
        self.request.wrapped_response = self.request.response

        res = renderer(context2, self.request).body
        self.assertEqual('<html><div>View: test</div>\n</html>\n', text_(res))

    def test_layout_chain_parent_notfound(self):
        ptah.layout.register('', context=Context, parent='page',
                             renderer='ptah:tests/test-layout.pt')
        self.init_ptah()

        root = Root()
        context = Context(root)

        from ptah.layout import LayoutRenderer
        renderer = LayoutRenderer('')
        self.request.wrapped_body = 'View: test'
        self.request.wrapped_response = self.request.response

        res = renderer(context, self.request).body
        self.assertTrue('<div>View: test</div>' in text_(res))

    def test_layout_for_route(self):
        from ptah.layout import query_layout, LayoutRenderer

        self.config.add_route('test-route', '/test/', use_global_views=False)
        ptah.layout.register('test', route_name='test-route')
        self.init_ptah()

        layout, context = query_layout(None, Context(), self.request, 'test')
        self.assertIsNone(layout)

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')
        self.request.request_iface = request_iface

        layout = query_layout(None, Context(), self.request, 'test')
        self.assertIsNotNone(layout)

    def test_layout_for_route_global_views(self):
        from ptah.layout import query_layout

        self.config.add_route('test-route', '/test/', use_global_views=False)
        ptah.layout.register('test', use_global_views=True)
        self.init_ptah()

        request_iface = self.registry.getUtility(
            IRouteRequest, name='test-route')
        self.request.request_iface = request_iface

        layout, context = query_layout(object(), object(), self.request, 'test')
        self.assertIsNotNone(layout)

    def test_layout_root(self):
        from ptah.layout import query_layout

        class Root1(object):
            pass

        class Root2(object):
            pass

        ptah.layout.register('test', root=Root1)
        self.init_ptah()

        layout, context = query_layout(Root1(), object(), self.request, 'test')
        self.assertIsNotNone(layout)

        layout, context = query_layout(Root2(), object(), self.request, 'test')
        self.assertIsNone(layout)

    def test_layout_chain_multi_level(self):
        class Layout1(ptah.View):
            """ """
        class Layout2(ptah.View):
            """ """
        class Layout3(ptah.View):
            """ """

        class Context1(object):
            """ """
        class Context2(object):
            """ """
        class Context3(object):
            """ """

        ptah.layout.register('l1', view=Layout1, context=Context1)
        ptah.layout.register('l1', view=Layout2, context=Context2, parent='l1')
        ptah.layout.register('l3', view=Layout3, context=Context3, parent='l1')
        self.init_ptah()

        root = Root()
        context1 = Context1()
        context2 = Context2()
        context3 = Context3()

        context1.__parent__ = root
        context2.__parent__ = context1
        context3.__parent__ = context2

        from ptah.layout import query_layout_chain
        chain = query_layout_chain(root, context1, self.request, 'l1')

        self.assertEqual(len(chain), 1)
        self.assertIs(chain[0][0].original, Layout1)

        chain = query_layout_chain(root, context2, self.request, 'l1')

        self.assertEqual(len(chain), 2)
        self.assertIs(chain[0][0].original, Layout2)
        self.assertIs(chain[1][0].original, Layout1)

        chain = query_layout_chain(root, context3, self.request, 'l3')

        self.assertEqual(len(chain), 3)
        self.assertIs(chain[0][0].original, Layout3)
        self.assertIs(chain[1][0].original, Layout2)
        self.assertIs(chain[2][0].original, Layout1)


class Context(object):

    def __init__(self, parent=None):
        self.__parent__ = parent


class Context2(object):

    def __init__(self, parent=None):
        self.__parent__ = parent


class Root(Context):
    pass



""" formatters """
import pytz
import translationstring
from datetime import datetime, timedelta
from pyramid.i18n import get_localizer
from pyramid.compat import text_type
from pyramid.threadlocal import get_current_request

import ptah
from ptah import config

_ = translationstring.TranslationStringFactory('ptah')

ID_FORMATTER = 'ptah:formatter'


def formatter(name):
    info = config.DirectiveInfo()

    def wrapper(func):
        discr = (ID_FORMATTER, name)

        intr = config.Introspectable(ID_FORMATTER, discr, name, ID_FORMATTER)

        intr['name'] = name
        intr['callable'] = func
        intr['description'] = func.__doc__
        intr['codeinfo'] = info.codeinfo

        info.attach(
            config.Action(
                lambda config, name, func:
                    config.get_cfg_storage(ID_FORMATTER).update({name: func}),
                (name, func), discriminator=discr, introspectables=(intr,))
            )
        return func

    return wrapper


class FormatImpl(dict):

    def __getattr__(self, name):
        try:
            return config.get_cfg_storage(ID_FORMATTER)[name]
        except KeyError:
            raise AttributeError(name)

format = FormatImpl()


@formatter('datetime')
def datetime_formatter(value, tp='medium', request=None):
    """DateTime formatter

    Short::

      >> dt = datetime(2011, 2, 6, 10, 35, 45, 80, pytz.UTC)

      >> ptah.format.datetime(dt, 'short')
      '02/06/11 04:35 AM'


    Medium::

      >> ptah.format.datetime(dt, 'medium')
      'Feb 06, 2011 04:35 AM'

    Long::

      >> ptah.format.datetime(dt, 'long')
      'February 06, 2011 04:35 AM -0600'

    Full::

      >> ptah.format.datetime(dt, 'full')
      'Sunday, February 06, 2011 04:35:45 AM CST'

    """
    if not isinstance(value, datetime):
        return value

    FORMAT = ptah.get_settings('format', request)

    tz = FORMAT['timezone']
    if value.tzinfo is None:
        value = datetime(value.year, value.month, value.day, value.hour,
                         value.minute, value.second, value.microsecond,
                         pytz.utc)

    value = value.astimezone(tz)

    format = '%s %s' % (FORMAT['date_%s' % tp], FORMAT['time_%s' % tp])
    return text_type(value.strftime(str(format)))


@formatter('timedelta')
def timedelta_formatter(value, type='short', request=None):
    """Timedelta formatter

    Full format::

      >> td = timedelta(hours=10, minutes=5, seconds=45)
      >> ptah.format.timedelta(td, 'full')
      '10 hour(s) 5 min(s) 45 sec(s)'

    Seconds::

      >> ptah.format.timedelta(td, 'seconds')
      '36345.0000'


    Default::

      >> ptah.format.timedelta(td)
      '10:05:45'

    """
    if not isinstance(value, timedelta):
        return value

    if request is None:
        request = get_current_request()

    if type == 'full':
        hours = value.seconds // 3600
        hs = hours * 3600
        mins = (value.seconds - hs) // 60
        ms = mins * 60
        secs = value.seconds - hs - ms
        frm = []
        translate = get_localizer(request).translate

        if hours:
            frm.append(translate(
                    '${hours} hour(s)', 'ptah.view', {'hours': hours}))
        if mins:
            frm.append(translate(
                    '${mins} min(s)', 'ptah.view', {'mins': mins}))
        if secs:
            frm.append(translate(
                    '${secs} sec(s)', 'ptah.view', {'secs': secs}))

        return ' '.join(frm)

    elif type == 'medium':
        return str(value)
    elif type == 'seconds':
        s = value.seconds + value.microseconds / 1000000.0
        return '%2.4f' % s
    else:
        return str(value).split('.')[0]


_size_types = {
    'b': (1.0, 'B'),
    'k': (1024.0, 'KB'),
    'm': (1024.0*1024.0, 'MB'),
    'g': (1024.0*1024.0*1024.0, 'GB'),
}


@formatter('size')
def size_formatter(value, type='k', request=None):
    """Size formatter

    bytes::

        >> v = 1024
        >> ptah.format.size(v, 'b')
        '1024 B'

    kylobytes::

        >> ptah.format.size(v, 'k')
        '1.00 KB'

    megabytes::

        >> ptah.format.size(1024*768, 'm')
        '0.75 MB'

        >> ptah.format.size(1024*768*768, 'm')
        '576.00 MB'

    terabytes::

        >> ptah.format.size(1024*768*768, 'g')
        '0.56 GB'

    """
    if not isinstance(value, (int, float)):
        return value

    f, t = _size_types.get(type, (1024.0, 'KB'))

    if t == 'B':
        return '%.0f %s' % (value / f, t)

    return '%.2f %s' % (value / f, t)



""" mail settings """
import ptah
import os.path
import itertools
from email import encoders
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.nonmultipart import MIMENonMultipart
from email.utils import formatdate, formataddr
from email.header import make_header

from pyramid import renderers
from pyramid.compat import bytes_


class MailGenerator(object):
    """ mail generator """

    def __init__(self, context):
        self._headers = {}
        self.context = context

    def _add_header(self, header, value, encode=False):
        self._headers[header] = (header, value, encode)

    def set_headers(self, message):
        charset = str(self.context.charset)

        extra = list(self.context.get_headers())
        for key, val, encode in itertools.chain(self._headers.values(), extra):
            if encode:
                message[key] = make_header(((val, charset),))
            else:
                message[key] = val

    def get_message(self):
        """ render a mail template """
        context = self.context

        charset = str(context.charset)
        contentType = context.content_type

        mail_body = context.render()
        maintype, subtype = contentType.split('/')

        return MIMEText(mail_body, subtype, charset)

    def get_attachments(self):
        attachments = []

        # attach files
        for data, content_type, filename, disposition in \
                self.context.get_attachments():
            maintype, subtype = content_type.split('/')

            msg = MIMENonMultipart(maintype, subtype)

            msg.set_payload(data)
            if filename:
                msg['Content-Id']=bytes_('<{0}@ptah>'.format(filename),'utf-8')
                msg['Content-Disposition']=bytes_('{0}; filename="{1}"'.format(
                    disposition, filename), 'utf-8')

            encoders.encode_base64(msg)
            attachments.append(msg)

        return attachments

    def message(self, multipart_format='mixed', *args, **kw):
        context = self.context

        # generate message
        message = self.get_message()

        # generate attachments
        attachments = self.get_attachments()
        if attachments:
            # create multipart message
            root = MIMEMultipart(multipart_format)

            # insert headers
            self.set_headers(root)

            # create message with attachments
            related = MIMEMultipart('related')
            related.attach(message)

            for attach in attachments:
                disposition = attach['Content-Disposition']\
                              .decode('utf-8').split(';')[0]
                if disposition == 'attachment':
                    root.attach(attach)
                else:
                    related.attach(attach)

            root.attach(related)
            message = root

        # alternative
        alternatives = self.context.get_alternative()
        if alternatives:
            mainmessage = MIMEMultipart('alternative')
            mainmessage.attach(message)

            for msg in alternatives:
                mainmessage.attach(MailGenerator(msg).message(
                        multipart_format, *args, **kw))

            message = mainmessage

        # default headers
        self._add_header('Subject', context.subject, True)

        self.set_headers(message)
        return message

    def __call__(self, multipart_format='mixed', *args, **kw):
        context = self.context
        message = self.message(multipart_format, *args, **kw)

        message['Date'] = formatdate()
        message['Message-ID'] = context.message_id

        if not message.get('To') and context.to_address:
            message['To'] = context.to_address

        if not message.get('From') and context.from_address:
            message['From'] = formataddr(
                (context.from_name, context.from_address))

        return message


class MailTemplate(object):
    """ mail template with base features """

    subject = ''
    charset = 'utf-8'
    content_type = 'text/plain'
    message_id = None
    template = None

    from_name = ''
    from_address = ''
    to_address = ''
    return_address = ''
    errors_address = ''

    def __init__(self, context, request, **kwargs):
        self.__dict__.update(kwargs)

        self.context = context
        self.request = request
        self.cfg = ptah.get_settings(ptah.CFG_ID_PTAH, request.registry)

        self._files = []
        self._headers = {}
        self._alternative = []

    def add_header(self, header, value, encode=False):
        self._headers[header] = (header, value, encode)

    def has_header(self, header):
        header = header.lower()
        for key in self._headers.keys():
            if key.lower() == header:
                return True

        return False

    def get_headers(self):
        return self._headers.values()

    def add_attachment(self, file_data, content_type,
                       filename, disposition='attachment'):
        self._files.append((file_data, content_type,
                            wrap_filename(filename), disposition))

    def get_attachments(self):
        return self._files

    def add_alternative(self, template):
        self._alternative.append(template)

    def get_alternative(self):
        return self._alternative

    def update(self):
        if not self.from_name:
            self.from_name = self.cfg['email_from_name']
        if not self.from_address:
            self.from_address = self.cfg['email_from_address']

    def render(self):
        kwargs = {'view': self,
                  'context': self.context,
                  'request': self.request}

        return renderers.render(self.template, kwargs, self.request)

    def send(self, email=None, mailer=None, **kw):
        if email:
            self.to_address = email

        message = self(**kw)

        if mailer is None:
            mailer = self.cfg.get('Mailer')

        if mailer is not None:
            mailer.send(message['from'], message['to'], message)

    def __call__(self, **kw):
        for key, value in kw.items():
            if type(value) is tuple:
                self.add_header(key, value[0], value[1])
            else:
                self.add_header(key, value)

        self.update()
        return MailGenerator(self)()


def wrap_filename(f_name):
    dir, f_name = os.path.split(f_name)
    f_name = f_name.split('\\')[-1].split('/')[-1]
    for key in '~,\'':
        f_name = f_name.replace(key, '_')

    return f_name



import os
import ptah
import shutil
import tempfile
from pyramid.config import ConfigurationConflictError


class TestRegisterMigration(ptah.PtahTestCase):

    _init_ptah = False

    def test_register(self):
        from ptah.migrate import MIGRATION_ID

        ptah.register_migration(
            'test', 'ptah.tests:migrations', 'Ptah database migration')
        self.init_ptah()

        st = ptah.get_cfg_storage(MIGRATION_ID)

        self.assertIn('test', st)
        self.assertEqual(st['test'], 'ptah.tests:migrations')

    def test_register_conflict(self):
        from ptah.migrate import MIGRATION_ID

        ptah.register_migration(
            'test', 'ptah.tests:migrations', 'Ptah database migration')
        ptah.register_migration(
            'test', 'ptah.tests:migrations', 'Ptah database migration')

        self.assertRaises(ConfigurationConflictError, self.init_ptah)


class TestPyramidDirective(ptah.PtahTestCase):

    def setUp(self):
        from ptah import migrate

        self._pkgs = []
        def upgrade(pkg):
            self._pkgs.append(pkg)

        self.orig_upgrade = migrate.upgrade
        migrate.upgrade = upgrade

        super(TestPyramidDirective, self).setUp()

    def tearDown(self):
        super(TestPyramidDirective, self).tearDown()

        from ptah import migrate
        migrate.upgrade = self.orig_upgrade

    def test_pyramid_directive(self):
        from pyramid.config import Configurator

        config = Configurator()
        config.include('ptah')

        self.assertTrue(hasattr(config, 'ptah_migrate'))

    def test_directive_execution(self):
        from pyramid.config import Configurator

        config = Configurator()
        config.include('ptah')
        config.ptah_migrate()
        config.commit()

        self.assertEqual(self._pkgs, ['ptah'])


class TestScriptDirectory(ptah.PtahTestCase):

    _init_ptah = False

    def test_unknown_package(self):
        self.init_ptah()

        from ptah.migrate import ScriptDirectory

        self.assertRaises(ValueError, ScriptDirectory, 'unknown')

    def test_normal(self):
        self.init_ptah()

        from ptah.migrate import ScriptDirectory

        script = ScriptDirectory('ptah')

        self.assertEqual(
            script.dir,
            os.path.join(os.path.dirname(ptah.__file__), 'scripts'))

        self.assertEqual(
            script.versions,
            os.path.join(os.path.dirname(ptah.__file__), 'migrations'))

    def test_doesnt_exist(self):
        from alembic.util import CommandError
        from ptah.migrate import ScriptDirectory

        ptah.register_migration('test', 'ptah:unknown_migrations')
        self.init_ptah()

        self.assertRaises(CommandError, ScriptDirectory, 'test')


class BaseScript(ptah.PtahTestCase):

    def setUp(self):
        from ptah import migrate

        self.dirs = dirs = {}

        class ScriptDirectory(migrate.ScriptDirectory):

            def __init__(self, pkg):
                if pkg in dirs:
                    dir = dirs[pkg]
                else:
                    dir = tempfile.mkdtemp()
                    dirs[pkg] = dir

                self.dir = os.path.join(
                    os.path.dirname(ptah.__file__), 'scripts')
                self.versions = dir

        self.orig_ScriptDirectory = migrate.ScriptDirectory
        migrate.ScriptDirectory = ScriptDirectory

        super(BaseScript, self).setUp()

    def tearDown(self):
        super(BaseScript, self).tearDown()

        for dir in self.dirs.values():
            shutil.rmtree(dir)

        from ptah import migrate
        migrate.ScriptDirectory = self.orig_ScriptDirectory


class TestRevision(BaseScript):

    def test_revision_default(self):
        from ptah.migrate import revision

        rev = revision('test', message='Test message')
        self.assertIn('{0}.py'.format(rev), os.listdir(self.dirs['test']))

    def test_revision_custom(self):
        from ptah.migrate import revision

        rev = revision('test', rev='001', message='Test message')
        self.assertEqual(rev, '001')
        self.assertIn('001.py', os.listdir(self.dirs['test']))

        self.assertRaises(
            KeyError, revision, 'test', rev='001')


class TestUpdateVersions(BaseScript):

    _init_ptah = False

    def test_update_force(self):
        from ptah.migrate import Version, revision, update_versions

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev = revision('test', message='Test message')

        update_versions(self.registry)

        session = ptah.get_session()

        versions = dict((v.package, v.version_num)
                        for v in session.query(Version).all())
        self.assertNotIn('test', versions)

    def test_update_versions(self):
        from ptah.migrate import Version, revision, update_versions

        ptah.register_migration('test', 'ptah.tests:migrations')
        self.init_ptah()

        rev = revision('test', message='Test message')

        update_versions(self.registry)

        session = ptah.get_session()

        versions = dict((v.package, v.version_num)
                        for v in session.query(Version).all())
        self.assertIn('test', versions)
        self.assertEqual(versions['test'], rev)

    def test_update_version_exists(self):
        from ptah.migrate import Version, revision, update_versions

        ptah.register_migration('test', 'ptah.tests:migrations')
        self.init_ptah()

        rev = revision('test', message='Test message')

        session = ptah.get_session()

        session.add(Version(package='test', version_num='123'))
        session.flush()

        update_versions(self.registry)

        versions = dict((v.package, v.version_num)
                        for v in session.query(Version).all())
        self.assertIn('test', versions)
        self.assertEqual(versions['test'], '123')


class TestUpgrade(BaseScript):

    _init_ptah = False

    def test_upgrade_to_rev(self):
        from ptah.migrate import Version, revision, upgrade

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev1 = revision('test', message='Test message1')
        rev2 = revision('test', message='Test message2')

        upgrade('test:%s'%rev1)

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev1)

    def test_upgrade_to_head(self):
        from ptah.migrate import Version, revision, upgrade

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev1 = revision('test', message='Test message1')
        rev2 = revision('test', message='Test message2')

        upgrade('test:head')

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev2)

    def test_upgrade_to_head_by_default(self):
        from ptah.migrate import Version, revision, upgrade

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev1 = revision('test', message='Test message1')
        rev2 = revision('test', message='Test message2')

        upgrade('test')

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev2)

    def test_upgrade_to_same_rev(self):
        from ptah.migrate import Version, revision, upgrade

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev1 = revision('test', message='Test message1')
        rev2 = revision('test', message='Test message2')

        upgrade('test')

        upgrade('test')

        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev2)

    def test_upgrade_in_two_steps(self):
        from ptah.migrate import Version, revision, upgrade

        ptah.register_migration(
            'test', 'ptah.tests:migrations', force=True)
        self.init_ptah()

        rev1 = revision('test', message='Test message1')
        rev2 = revision('test', message='Test message2')

        upgrade('test:%s'%rev1)
        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev1)

        upgrade('test')
        versions = dict((v.package, v.version_num)
                        for v in ptah.get_session().query(Version).all())
        self.assertEqual(versions['test'], rev2)

    def test_startup_check_version(self):
        from ptah.migrate import Version, revision, check_version

        ptah.register_migration(
            'test1', 'ptah.tests1:migrations')
        ptah.register_migration(
            'test2', 'ptah.tests2:migrations')
        self.init_ptah()

        rev1 = revision('test1', message='Test message1')
        rev2 = revision('test2', message='Test message2')

        session = ptah.get_session()
        session.add(Version(package='test1', version_num=rev1))
        session.add(Version(package='test2', version_num=rev2))
        session.flush()

        exc = None
        try:
            check_version(None)
        except SystemExit as err: # pragma: no cover
            exc = err

        self.assertIsNone(exc)

    def test_startup_check_version_exit(self):
        from ptah.migrate import Version, revision, check_version

        ptah.register_migration(
            'test1', 'ptah.tests1:migrations')
        ptah.register_migration(
            'test2', 'ptah.tests2:migrations')
        self.init_ptah()

        rev1 = revision('test1', message='Test message1')
        rev2 = revision('test2', message='Test message2')

        exc = None
        try:
            check_version(None)
        except SystemExit as err: # pragma: no cover
            exc = err

        self.assertIsInstance(exc, SystemExit)



import os
import shutil
import tempfile
from pyramid.compat import bytes_
from zope.interface.interface import InterfaceClass
from zope.interface.interfaces import IObjectEvent

import ptah
from ptah.settings import Settings
from ptah.settings import SETTINGS_OB_ID
from ptah.testing import PtahTestCase


def get_settings_ob():
    return ptah.config.get_cfg_storage(SETTINGS_OB_ID, default_factory=Settings)


class BaseTesting(PtahTestCase):

    _init_ptah = False

    def init_ptah(self, initsettings=False, *args, **kw):
        self.config.include('ptah')
        self.config.scan(self.__class__.__module__)
        self.config.commit()
        self.config.autocommit = True

        if initsettings:
            self.config.ptah_init_settings()


class TestSettingsResolver(BaseTesting):

    def test_settings_uri_resolver(self):
        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group', node,
            title = 'Section title',
            description = 'Section description',
            )
        self.init_ptah()

        grp1 = ptah.get_settings('group', self.registry)
        self.assertEqual(grp1.__uri__, 'settings:group')

        grp2 = ptah.resolve('settings:group')
        self.assertIs(grp1, grp2)


class TestSettings(BaseTesting):

    def test_settings_no_default(self):
        field = ptah.form.TextField('node')

        self.assertRaises(
            ptah.config.StopException,
            ptah.register_settings,
            'group1', field,
            title = 'Section title',
            description = 'Section description',
            )

    def test_settings_group_basics(self):
        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )
        self.init_ptah()

        group = ptah.get_settings('group1', self.registry)
        self.assertEqual(group.keys(), ['node'])
        self.assertEqual(group.items(), [('node', 'test')])

        group.update({'node': '12345'})
        self.assertEqual(group.get('node'), '12345')

    def test_settings_group_uninitialized(self):
        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )
        self.init_ptah()

        group = ptah.get_settings('group1', self.registry)
        self.assertEqual(group.get('node'), 'test')

    def test_settings_group_extract(self):
        node1 = ptah.form.TextField(
            'node1', default = 'test1')

        node2 = ptah.form.TextField(
            'node2', default = 'test2')

        ptah.register_settings('group', node1, node2)
        self.init_ptah()

        group = ptah.get_settings('group', self.registry)

        data, errors = group.extract({'group.node1': 'test-extract'})

        self.assertEqual(data['node1'], 'test-extract')
        self.assertEqual(data['node2'], 'test2')

        group['node2'] = 'value'
        data, errors = group.extract({'group.node1': 'test-extract'})
        self.assertEqual(data['node2'], 'value')

    def test_settings_get_settings_pyramid(self):
        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )

        self.init_ptah()

        grp = self.config.ptah_get_settings('group1')
        self.assertIsNotNone(grp)
        self.assertEqual(grp.__name__, 'group1')
        self.assertIn(node, grp.__fields__.values())

    def test_settings_register_simple(self):
        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group1', node,
            title = 'Section title',
            description = 'Section description',
            )

        self.init_ptah()

        group = ptah.get_settings('group1', self.registry)

        self.assertEqual(len(group.__fields__), 1)
        self.assertIn(node, group.__fields__.values())
        self.assertEqual(group['node'], 'test')
        self.assertRaises(
            KeyError,
            group.__getitem__, 'unknown')

        group.node = 'test2'
        self.assertFalse(group.node == group['node'])

    def test_settings_group_validation(self):
        def validator(node, appstruct):
            raise ptah.form.Invalid(node['node'], 'Error')

        node = ptah.form.TextField(
            'node',
            default = 'test')

        ptah.register_settings(
            'group2', node, validator=validator)

        self.init_ptah()

        group = ptah.get_settings('group2', self.registry)

        data, err = group.extract({'group2.node': 'value'})

        self.assertEqual(err.msg, {'group2': ['Error']})

    def test_settings_group_multiple_validation(self):
        def validator1(fs, appstruct):
            raise ptah.form.Invalid(fs['node1'], 'Error1')

        def validator2(fs, appstruct):
            raise ptah.form.Invalid(fs['node2'], 'Error2')

        node1 = ptah.form.TextField(
            'node1',
            default = 'test')

        node2 = ptah.form.TextField(
            'node2',
            default = 'test')

        ptah.register_settings(
            'group3', node1, node2, validator=(validator1, validator2))

        self.init_ptah()

        group = ptah.get_settings('group3', self.registry)
        data, err = group.extract({
            'group3.node1': 'value',
            'group3.node2': 'value'})

        self.assertEqual(err.msg, {'group3': ['Error1', 'Error2']})

    def test_settings_export(self):
        field1 = ptah.form.TextField(
            'node1',
            default = 'test')

        field2 = ptah.form.TextField(
            'node2',
            default = 'test1')

        ptah.register_settings('group4', field1, field2)
        self.init_ptah(initsettings=True)

        settings = get_settings_ob()

        # changed settings
        self.assertEqual(settings.export(), {})

        # default settings
        data = settings.export(default=True)
        self.assertIn('group4.node1', data)
        self.assertIn('group4.node2', data)
        self.assertEqual(data['group4.node1'], '"test"')
        self.assertEqual(data['group4.node2'], '"test1"')

        # changed settings
        group = ptah.get_settings('group4', self.registry)

        group['node2'] = 'changed'
        data = dict(settings.export())
        self.assertEqual(data['group4.node2'], '"changed"')

    def _create_default_group(self):
        node1 = ptah.form.TextField(
            'node1',
            default = 'default1')

        node2 = ptah.form.IntegerField(
            'node2',
            default = 10)

        ptah.register_settings('group', node1, node2)
        self.init_ptah(initsettings=True)

        return ptah.get_settings('group', self.registry)

    def test_settings_load_rawdata(self):
        group = self._create_default_group()

        get_settings_ob().init(self.config, {'group.node1': 'val1'})

        # new value
        self.assertEqual(group['node1'], 'val1')

        # default value
        self.assertEqual(group['node2'], 10)

    def test_settings_load_rawdata_and_change_defaults(self):
        group = self._create_default_group()

        # change defaults
        get_settings_ob().init(self.config, {'group.node2': '30'})

        # new values
        self.assertEqual(group['node1'], 'default1')
        self.assertEqual(group['node2'], 30)

        self.assertEqual(group.__fields__['node1'].default, 'default1')
        self.assertEqual(group.__fields__['node2'].default, 30)

    def test_settings_load_rawdata_with_errors_in_rawdata(self):
        group = self._create_default_group()

        self.assertRaises(
            ptah.config.StopException,
            get_settings_ob().init,
            self.config, {10: 'value'})

    def test_settings_load_defaults_rawdata_with_errors_in_values(self):
        node = ptah.form.LinesField(
            'node1',
            default = ())

        ptah.register_settings('group', node)
        self.init_ptah(initsettings=True)

        self.assertRaises(
            ptah.config.StopException,
            get_settings_ob().init,
            self.config,
            {'group.node1': '1,l'})

    def test_settings_init_with_no_loader_with_defaults(self):
        group = self._create_default_group()

        get_settings_ob().init(self.config,
                               {'group.node1': 'new-default',
                                'group.node2': '50'})

        self.assertEqual(group['node1'], 'new-default')
        self.assertEqual(group['node2'], 50)

        self.assertEqual(group.__fields__['node1'].default, 'new-default')
        self.assertEqual(group.__fields__['node2'].default, 50)


class TestSettingsInitialization(BaseTesting):

    def setUp(self):
        BaseTesting.setUp(self)
        self.dir = tempfile.mkdtemp()

    def tearDown(self):
        BaseTesting.tearDown(self)
        shutil.rmtree(self.dir)

    def test_settings_initialize_events(self):
        from ptah.settings import init_settings

        self.init_ptah()

        sm = self.config.registry

        events = []

        def h1(ev):
            events.append(ev)

        def h2(ev):
            events.append(ev)

        sm.registerHandler(h1, (ptah.events.SettingsInitializing,))
        sm.registerHandler(h2, (ptah.events.SettingsInitialized,))

        init_settings(self.config, {})

        self.assertTrue(isinstance(events[0], ptah.events.SettingsInitializing))
        self.assertTrue(isinstance(events[1], ptah.events.SettingsInitialized))

        self.assertTrue(events[0].config is self.config)
        self.assertTrue(events[1].config is self.config)

    def test_settings_initialize_events_exceptions(self):
        from ptah.settings import init_settings

        self.init_ptah()

        sm = self.config.registry

        events = []
        err_tp = TypeError()

        def h1(ev):
            raise err_tp

        sm.registerHandler(h1, (ptah.events.SettingsInitializing,))

        err = None
        try:
            init_settings(self.config, {})
        except Exception as exc:
            err = exc

        self.assertIsInstance(err, ptah.config.StopException)
        self.assertIs(err.exc, err_tp)

    def test_settings_initialize_only_once(self):
        from ptah.settings import init_settings

        self.init_ptah()
        init_settings(self.config, {})

        self.assertRaises(
            RuntimeError, init_settings, self.config, {})

    def test_settings_initialize_load_defaults(self):
        from ptah.settings import init_settings

        node1 = ptah.form.TextField(
            'node1',
            default = 'default1')

        node2 = ptah.form.TextField(
            'node2',
            default = 10)

        ptah.register_settings('group', node1, node2)
        self.init_ptah()

        init_settings(self.config, None)

        group = ptah.get_settings('group', self.request.registry)
        self.assertEqual(group['node1'], 'default1')
        self.assertEqual(group['node2'], 10)

    def test_settings_initialize_load_preparer(self):
        from ptah.settings import init_settings

        node1 = ptah.form.TextField(
            'node',
            default = 'default1',
            preparer = lambda s: s.lower())

        ptah.register_settings('group', node1)
        self.init_ptah()

        init_settings(self.config, {'group.node': 'Test'})

        group = ptah.get_settings('group', self.request.registry)
        self.assertEqual(group['node'], 'test')

    def test_settings_initialize_load_partly_defaults(self):
        from ptah.settings import init_settings

        node1 = ptah.form.TextField(
            'node1',
            default = 'default1')

        node2 = ptah.form.TextField(
            'node2',
            default = 10)

        ptah.register_settings('group', node1, node2)
        self.init_ptah()

        init_settings(self.config, {'group.node1': 'setting from ini'})

        group = ptah.get_settings('group', self.request.registry)
        self.assertEqual(group['node1'], 'setting from ini')
        self.assertEqual(group['node2'], 10)

    def test_settings_initialize_load_settings_include(self):
        from ptah.settings import init_settings

        path = os.path.join(self.dir, 'settings.cfg')
        f = open(path, 'wb')
        f.write(bytes_('[DEFAULT]\ngroup.node1 = value\n\n','ascii'))
        f.close()

        node1 = ptah.form.TextField(
            'node1',
            default = 'default1')

        node2 = ptah.form.IntegerField(
            'node2',
            default = 10)

        ptah.register_settings('group', node1, node2)
        self.init_ptah()

        init_settings(self.config, {'include': path})

        group = ptah.get_settings('group', self.request.registry)

        self.assertEqual(group['node1'], 'value')
        self.assertEqual(group['node2'], 10)


class TestDBSettingsBase(PtahTestCase):

    def _make_grp(self):
        node1 = ptah.form.TextField(
            'node1',
            default = 'test')

        node2 = ptah.form.IntegerField(
            'node2',
            default = 50)

        ptah.register_settings(
            'group', node1, node2,
            title = 'Section title',
            description = 'Section description',
            )
        self.init_ptah()

        return ptah.get_settings('group', self.registry)


class TestDBSettings(TestDBSettingsBase):

    _init_ptah = False

    def test_settings_updatedb(self):
        grp = self._make_grp()
        grp.updatedb(node1 = 'new text',
                     node2 = 65)

        self.assertEqual(grp['node1'], 'new text')
        self.assertEqual(grp['node2'], 65)

        from ptah.settings import SettingRecord
        Session = ptah.get_session()

        res = {}
        for rec in Session.query(SettingRecord):
            res[rec.name] = rec.value

        self.assertIn('group.node1', res)
        self.assertIn('group.node2', res)
        self.assertEqual(res['group.node1'], '"new text"')
        self.assertEqual(res['group.node2'], '65')

    def test_settings_updatedb_unknown(self):
        grp = self._make_grp()
        grp.updatedb(node1 = 'new text',
                     node2 = 65,
                     node3 = 500)

        self.assertEqual(grp['node3'], 500)

        from ptah.settings import SettingRecord
        Session = ptah.get_session()

        res = {}
        for rec in Session.query(SettingRecord):
            res[rec.name] = rec.value

        self.assertEqual(len(res), 2)
        self.assertNotIn('group.node3', res)

    def test_settings_updatedb_partial(self):
        grp = self._make_grp()
        grp.updatedb(node1 = 'new text')

        self.assertEqual(grp['node1'], 'new text')
        self.assertEqual(grp['node2'], 50)

        from ptah.settings import SettingRecord
        Session = ptah.get_session()

        res = {}
        for rec in Session.query(SettingRecord):
            res[rec.name] = rec.value

        self.assertEqual(len(res), 1)
        self.assertIn('group.node1', res)
        self.assertEqual(res['group.node1'], '"new text"')

    def test_settings_updatedb_set_default(self):
        grp = self._make_grp()
        grp.updatedb(node1 = 'new text', node2 = 65)
        grp.updatedb(node1 = 'new text 2', node2 = 50)

        from ptah.settings import SettingRecord
        Session = ptah.get_session()

        res = {}
        for rec in Session.query(SettingRecord):
            res[rec.name] = rec.value

        self.assertEqual(len(res), 1)
        self.assertIn('group.node1', res)
        self.assertEqual(res['group.node1'], '"new text 2"')

    def test_settings_updatedb_event(self):
        event_grp = []

        @ptah.config.subscriber(ptah.events.SettingsGroupModified)
        def handler(ev):
            event_grp.append(ev.object)

        grp = self._make_grp()
        grp.updatedb(node1 = 'new text', node2 = 65)

        self.assertIs(grp, event_grp[0])

    def test_settings_updatedb_load_from_db(self):
        grp = self._make_grp()
        grp.updatedb(node1 = 'new text',
                     node2 = 65)
        grp.clear()

        from ptah.settings import SettingRecord
        Session = ptah.get_session()

        settings = self.registry.__ptah_storage__[SETTINGS_OB_ID]
        settings.load({})

        self.assertEqual(grp['node1'], 'test')
        self.assertEqual(grp['node2'], 50)

        settings.load_fromdb()
        self.assertEqual(grp['node1'], 'new text')
        self.assertEqual(grp['node2'], 65)

    def test_settings_load_from_db_on_startup(self):
        grp = self._make_grp()

        grp.updatedb(node1 = 'new text',
                     node2 = 65)
        grp.clear()

        ptah.get_base().metadata.tables['ptah_db_versions'].drop()

        self.config.make_wsgi_app()
        self.assertEqual(grp['node1'], 'new text')
        self.assertEqual(grp['node2'], 65)


class TestDBSettings2(TestDBSettingsBase):

    _init_ptah = False
    _init_sqla = False

    def test_settings_load_from_db_on_startup_do_not_brake(self):
        from ptah.settings import SettingRecord
        ptah.get_base().metadata.drop_all()

        grp = self._make_grp()

        self.config.make_wsgi_app()

        self.assertEqual(grp['node1'], 'test')
        self.assertEqual(grp['node2'], 50)
        self.assertFalse(SettingRecord.__table__.exists())



import ptah
from ptah import config
from ptah.testing import PtahTestCase
from pyramid.exceptions import ConfigurationConflictError


class TestUIAction(PtahTestCase):

    _init_ptah = False

    def test_uiaction(self):
        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 1')
        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)

        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['id'], 'action1')

    def test_uiaction_conflicts(self):
        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 1')
        ptah.uiaction(Content, 'action1', 'Action 1')
        self.assertRaises(ConfigurationConflictError, self.init_ptah)

    def test_uiaction_url(self):

        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 1', action='test.html')
        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(actions[0]['url'], 'http://example.com/test.html')

    def test_uiaction_absolute_url(self):

        class Content(object):
            __name__ = ''

        ptah.uiaction(
            Content, 'action1', 'Action 1', action='/content/about.html')
        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(actions[0]['url'],
                         'http://example.com/content/about.html')

    def test_uiaction_custom_url(self):

        class Content(object):
            __name__ = ''

        def customAction(content, request):
            return 'http://github.com/ptahproject'

        ptah.uiaction(Content, 'action1', 'Action 1', action=customAction)

        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(actions[0]['url'], 'http://github.com/ptahproject')

    def test_uiaction_condition(self):

        class Content(object):
            __name__ = ''

        allow = False
        def condition(content, request):
            return allow

        ptah.uiaction(
            Content, 'action1', 'Action 1',
            action='test.html', condition=condition)

        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 0)

        allow = True
        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 1)

    def test_uiaction_permission(self):

        class Content(object):
            __name__ = ''

        allow = False
        def check_permission(permission, content, request=None, throw=False):
            return allow

        ptah.uiaction(Content, 'action1', 'Action 1', permission='View')

        self.init_ptah()

        orig_cp = ptah.check_permission
        ptah.check_permission = check_permission

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 0)

        allow = True
        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 1)

        ptah.check_permission = orig_cp

    def test_uiaction_sort_weight(self):

        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'view', 'View', sort_weight=1.0)
        ptah.uiaction(Content, 'action', 'Action', sort_weight=2.0)

        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)

        self.assertEqual(actions[0]['id'], 'view')
        self.assertEqual(actions[1]['id'], 'action')

    def test_uiaction_userdata(self):

        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'view', 'View', testinfo='test')

        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)

        self.assertEqual(actions[0]['data'], {'testinfo': 'test'})

    def test_uiaction_category(self):
        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 1',
                      category='test')

        self.init_ptah()

        request = self.request

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 0)

        actions = ptah.list_uiactions(Content(), self.request, category='test')

        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['id'], 'action1')

    def test_uiaction_category_reg_fail(self):
        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 10', category='test')
        ptah.uiaction(Content, 'action1', 'Action 11', category='test')
        self.assertRaises(ConfigurationConflictError, self.init_ptah)

    def test_uiaction_category_reg(self):
        class Content(object):
            __name__ = ''

        ptah.uiaction(Content, 'action1', 'Action 10')
        ptah.uiaction(Content, 'action1', 'Action 11', category='test')

        self.init_ptah()

        actions = ptah.list_uiactions(Content(), self.request)
        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['title'], 'Action 10')

        actions = ptah.list_uiactions(Content(), self.request, category='test')

        self.assertEqual(len(actions), 1)
        self.assertEqual(actions[0]['title'], 'Action 11')



from pyramid import paster
from pyramid.config import global_registries
from pyramid.request import Request
from pyramid.interfaces import IRequestFactory
from pyramid.threadlocal import manager as threadlocal_manager


def get_app(config_uri):
    from pyramid.router import Router
    from pyramid.config import Configurator

    def make_wsgi_app(self): # pragma: no cover
        self.commit()
        global_registries.add(self.registry)
        return Router(self.registry)

    # ugly hack
    orig = Configurator.make_wsgi_app

    Configurator.make_wsgi_app = make_wsgi_app
    app = paster.get_app(config_uri)
    Configurator.make_wsgi_app = orig
    return app


def bootstrap(config_uri):
    import ptah
    ptah.POPULATE = True

    app = get_app(config_uri)
    registry = global_registries.last

    request_factory = registry.queryUtility(IRequestFactory, default=Request)
    request = request_factory.blank('/')
    request.registry = registry

    threadlocals = {'registry':registry, 'request':request}
    threadlocal_manager.push(threadlocals)

    config_file = config_uri.split('#', 1)[0]
    paster.setup_logging(config_file)

    ptah.POPULATE = False
    return {'app':app, 'registry':registry, 'request': request}



""" assets libraries """
import ptah

# jQuery library
ptah.library(
    'jquery',
    path='ptah:static/jquery/jquery-1.7.1.min.js',
    type="js")

ptah.library(
    'jquery-ui',
    path='ptah:static/jquery/jquery-ui-1.8.16.min.js',
    type="js",
    require="jquery")

ptah.library(
    'jquery-ui',
    'ptah:static/jquery/jquery-ui.css',
    type='css')

# Bootstrap css
ptah.library(
    'bootstrap',
    path='ptah:static/bootstrap/bootstrap-1.4.0.min.css',
    type="css")

ptah.library(
    'bootstrap-js',
    path=('ptah:static/bootstrap/js/bootstrap-alerts.js',
          'ptah:static/bootstrap/js/bootstrap-buttons.js',
          'ptah:static/bootstrap/js/bootstrap-dropdown.js',
          'ptah:static/bootstrap/js/bootstrap-modal.js',
          'ptah:static/bootstrap/js/bootstrap-scrollspy.js',
          'ptah:static/bootstrap/js/bootstrap-tabs.js',
          'ptah:static/bootstrap/js/bootstrap-twipsy.js',
          'ptah:static/bootstrap/js/bootstrap-popover.js'),
    type="js",
    require="jquery")

# TinyMCE
ptah.library(
    "tiny_mce",
    path=('ptah:static/tiny_mce/tiny_mce.js',
          'ptah:static/tiny_mce/jquery.tinymce.js'),
    type="js",
    require='jquery')



""" ptah-migrate command """
from __future__ import print_function
import argparse
import textwrap
import alembic.config
import alembic.context
from pyramid.path import AssetResolver

import ptah
import ptah.migrate
from ptah import scripts
from ptah.populate import create_db_schema
from ptah.migrate import upgrade, revision
from ptah.migrate import Context, ScriptDirectory, MIGRATION_ID


def main():
    parser = argparse.ArgumentParser(description="ptah migrate")
    parser.add_argument('config', metavar='config',
                        help='ini config file')

    subparsers = parser.add_subparsers()

    # revision
    subparser = subparsers.add_parser(
        revision.__name__,
        help=revision.__doc__)
    subparser.add_argument('package', metavar='package',
                           help='package name')
    subparser.add_argument("-r", "--revision",
                           type=str, dest='revid',
                           help="Unique revision id")
    subparser.add_argument("-m", "--message",
                           type=str, dest='message',
                           help="Message string to use with 'revision'")
    subparser.set_defaults(cmd='revision')

    # current
    subparser = subparsers.add_parser(
        current.__name__,
        help=current.__doc__)
    subparser.add_argument('package', metavar='package',
                           nargs='*', help='package name')
    subparser.set_defaults(cmd='current')

    # upgrade
    subparser = subparsers.add_parser(
        upgrade.__name__,
        help=upgrade.__doc__)
    subparser.add_argument('package', metavar='package',
                           nargs='*', help='package name')
    subparser.set_defaults(cmd='upgrade')

    # history
    subparser = subparsers.add_parser(
        history.__name__,
        help=history.__doc__)
    subparser.add_argument('package', metavar='package',
                           nargs='*', help='package name')
    subparser.set_defaults(cmd='history')

    # list
    subparser = subparsers.add_parser(
        'list', help='List registered migrations.')
    subparser.set_defaults(cmd='list')

    # parse
    args = parser.parse_args()

    # bootstrap pyramid
    env = scripts.bootstrap(args.config)

    if args.cmd == 'current':
        print ('')
        if not args.package:
            args.package = ptah.get_cfg_storage(MIGRATION_ID).keys()

        for pkg in args.package:
            current(pkg)

    if args.cmd == 'revision':
        if args.revid:
            for ch in ',.;-':
                if ch in args.revid:
                    print ('Revision id contains forbidden characters')
                    ptah.shutdown()
                    return

        revision(args.package, args.revid, args.message)

    if args.cmd == 'upgrade':
        # create db schemas
        create_db_schema(env['registry'], False)

        for pkg in args.package:
            upgrade(pkg)

    if args.cmd == 'history':
        if not args.package:
            args.package = ptah.get_cfg_storage(MIGRATION_ID).keys()

        for pkg in args.package:
            history(pkg)

    if args.cmd == 'list':
        list_migrations(env['registry'])

    ptah.shutdown()


def history(pkg):
    """List changeset scripts in chronological order."""
    script = ptah.migrate.ScriptDirectory(pkg)
    print('')
    print (pkg)
    print ('='*len(pkg))
    for sc in script.walk_revisions():
        print('{0}: {1}'.format(sc.revision, sc.doc))


def current(pkg):
    """Display the current revision."""
    script = ptah.migrate.ScriptDirectory(pkg)

    def display_version(rev):
        rev = script._get_rev(rev)
        if rev is None:
            print ("Package '{0}' rev: None".format(pkg))
        else:
            print ("Package '{0}' rev: {1}{2} {3}".format(
                    pkg, rev.revision, '(head)' if rev.is_head else "",rev.doc))
        return []

    conn = ptah.get_base().metadata.bind.connect()

    alembic.context.Context = Context
    alembic.context.Context.pkg_name = pkg

    alembic.context._opts(
        alembic.config.Config(''),
        script,
        fn = display_version
    )

    alembic.context.configure(
        connection=conn,
        config=alembic.config.Config(''))

    with alembic.context.begin_transaction():
        alembic.context.run_migrations()


def list_migrations(registry):
    print ('')
    wrpTitle = textwrap.TextWrapper(
        initial_indent='* ',
        subsequent_indent='  ')

    wrpDesc = textwrap.TextWrapper(
        initial_indent='    ',
        subsequent_indent='    ')

    res = []
    for item in registry.introspector.get_category(MIGRATION_ID):
        intr = item['introspectable']
        res.append((intr['package'], intr['title'], intr['path']))

    for pkg, title, path in sorted(res):
        res = AssetResolver(pkg)
        print (wrpTitle.fill('{0}: {1}'.format(pkg, title)))
        print (wrpDesc.fill(path))
        print (wrpDesc.fill(res.resolve(path).abspath()))
        print ('')



from ptah import form, config
from ptah.testing import PtahTestCase
from pyramid.exceptions import ConfigurationConflictError


class TestFieldset(PtahTestCase):

    _init_ptah = False

    def test_decl(self):
        @form.field('my-field')
        class MyField(form.Field):
            pass

        self.init_ptah()
        self.assertIs(form.get_field_factory('my-field'), MyField)

    def test_decl_register(self):

        class MyField(form.Field):
            """ """

        form.field.register('my-field', MyField)

        self.init_ptah()
        self.assertIs(form.get_field_factory('my-field'), MyField)

    def test_decl_conflict(self):

        @form.field('my-field')
        class MyField(form.Field):
            pass

        @form.field('my-field')
        class MyField2(form.Field):
            pass

        self.assertRaises(ConfigurationConflictError, self.init_ptah)

    def test_decl_preview(self):

        @form.field('my-field')
        class MyField(form.Field):
            pass

        @form.fieldpreview(MyField)
        def preview(request):
            """ """

        self.init_ptah()

        from ptah.form.field import PREVIEW_ID
        previews = config.get_cfg_storage(PREVIEW_ID)

        self.assertIn(MyField, previews)
        self.assertIs(previews[MyField], preview)
        self.assertIs(form.get_field_preview(MyField), preview)



import ptah
from ptah import config
from ptah.testing import PtahTestCase
from pyramid.testing import DummyRequest
from pyramid.interfaces import IRouteRequest
from pyramid.httpexceptions import HTTPForbidden


class TestManageModule(PtahTestCase):

    _init_ptah = False

    def test_get_manage_url(self):
        from ptah.manage import get_manage_url
        self.init_ptah()

        self.request.application_url = 'http://example.com'
        self.assertEqual(get_manage_url(self.request),
                         'http://example.com/ptah-manage')

        self.request.application_url = 'http://example.com/'
        self.assertEqual(get_manage_url(self.request),
                         'http://example.com/ptah-manage')

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['manage'] = 'manage'

        self.assertEqual(get_manage_url(self.request),
                         'http://example.com/manage')

    def test_manage_module(self):
        from ptah.manage.manage import \
           module, MANAGE_ID, PtahModule, PtahManageRoute,\
           PtahAccessManager, set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        self.init_ptah()

        set_access_manager(PtahAccessManager())

        MODULES = config.get_cfg_storage(MANAGE_ID)
        self.assertIn('test-module', MODULES)

        self.assertRaises(HTTPForbidden, PtahManageRoute, self.request)

        ptah.auth_service.set_userid('test-user')

        self.assertRaises(HTTPForbidden, PtahManageRoute, self.request)

        def accessManager(id, request):
            return True

        set_access_manager(accessManager)

        route = PtahManageRoute(self.request)
        mod = route['test-module']
        self.assertIsInstance(mod, TestModule)
        self.assertTrue(mod.available())
        self.assertEqual(mod.__name__, 'test-module')
        self.assertEqual(mod.url(),'http://example.com/ptah-manage/test-module')

        self.assertRaises(KeyError, route.__getitem__, 'unknown')

    def test_manage_access_manager1(self):
        from ptah.manage.manage import PtahAccessManager
        self.init_ptah()

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ['*']

        self.assertTrue(PtahAccessManager()('test:user', self.request))

        cfg['managers'] = ['admin@ptahproject.org']

        self.assertFalse(PtahAccessManager()('test:user', self.request))

    def test_manage_access_manager2(self):
        from ptah.manage.manage import PtahAccessManager

        class Principal(object):
            id = 'test-user'
            login = 'admin@ptahproject.org'

        principal = Principal()

        @ptah.resolver('test')
        def principalResolver(uri):
            return principal

        self.init_ptah()

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['managers'] = ['admin@ptahproject.org']
        self.assertTrue(PtahAccessManager()('test:user', self.request))

    def test_manage_access_manager_role(self):
        from ptah.manage.manage import PtahAccessManager

        class Principal(object):
            id = 'test-user'
            login = 'admin@ptahproject.org'

        principal = Principal()

        @ptah.resolver('test')
        def principalResolver(uri):
            return principal

        self.init_ptah()

        orig_lr = ptah.get_local_roles

        def get_lr(userid, request, context):
            if userid == 'test:user':
                return ('Manager',)
            return ()

        ptah.get_local_roles = get_lr

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['manager_role'] = 'Manager'

        self.assertTrue(PtahAccessManager()('test:user', self.request))
        self.assertFalse(PtahAccessManager()('test:user2', self.request))

        ptah.get_local_roles = orig_lr

    def test_manage_check_access_no_manager(self):
        self.init_ptah()
        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        del cfg['access_manager']
        self.assertFalse(ptah.manage.check_access('test:user2', self.request))

    def test_manage_pyramid_directive(self):
        from pyramid.config import Configurator

        config = Configurator(autocommit=True)
        config.include('ptah')

        def access_manager():
            """ """

        config.ptah_init_manage(
            'test-manage',
            access_manager=access_manager,
            managers = ('manager',),
            manager_role = 'Manager',
            disable_modules = ('models',))

        cfg = config.ptah_get_settings(ptah.CFG_ID_PTAH)

        self.assertEqual(cfg['manage'], 'test-manage')
        self.assertEqual(cfg['managers'], ('manager',))
        self.assertEqual(cfg['manager_role'], 'Manager')
        self.assertEqual(cfg['disable_modules'], ('models',))
        self.assertIs(cfg['access_manager'], access_manager)

        iface = config.registry.getUtility(IRouteRequest, 'ptah-manage')
        self.assertIsNotNone(iface)

    def test_manage_pyramid_directive_default(self):
        from pyramid.config import Configurator
        from ptah.manage.manage import PtahAccessManager

        config = Configurator(autocommit=True)
        config.include('ptah')

        config.ptah_init_manage('test-manage')

        cfg = config.ptah_get_settings(ptah.CFG_ID_PTAH)
        self.assertIsInstance(cfg['access_manager'], PtahAccessManager)

    def test_manage_view(self):
        from ptah.manage.manage import \
            module, PtahModule, PtahManageRoute, ManageView, \
            set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        def accessManager(id, request):
            return True

        self.init_ptah()

        set_access_manager(accessManager)
        ptah.auth_service.set_userid('test-user')

        route = PtahManageRoute(self.request)

        v = ManageView(route, self.request)
        v.update()

        self.assertIsInstance(v.modules[-1], TestModule)

    def test_manage_view_unavailable(self):
        from ptah.manage.manage import \
            module, PtahModule, PtahManageRoute, ManageView, \
            set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

            def available(self):
                return False

        def accessManager(id, request):
            return True

        self.init_ptah()

        set_access_manager(accessManager)
        ptah.auth_service.set_userid('test-user')

        route = PtahManageRoute(self.request)

        v = ManageView(route, self.request)
        v.update()

        found = False
        for mod in v.modules:
            if isinstance(mod, TestModule): # pragma: no cover
                found = True

        self.assertFalse(found)

    def test_manage_traverse(self):
        from ptah.manage.manage import \
            module, PtahModule, PtahManageRoute, set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        def accessManager(id, request):
            return True

        self.init_ptah()

        set_access_manager(accessManager)
        ptah.auth_service.set_userid('test-user')

        route = PtahManageRoute(self.request)

        mod = route['test-module']
        self.assertIsInstance(mod, TestModule)

    def test_manage_disable_modules(self):
        from ptah.manage.manage import \
            module, PtahModule, PtahManageRoute, ManageView, set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        def accessManager(id, request):
            return True

        self.init_ptah()

        set_access_manager(accessManager)
        ptah.auth_service.set_userid('test-user')

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['disable_modules'] = ('test-module',)

        request = DummyRequest()
        route = PtahManageRoute(request)

        view = ManageView(route, request)
        view.update()

        for mod in view.modules:
            self.assertFalse(isinstance(mod, TestModule))

    def test_manage_disable_modules_traverse(self):
        from ptah.manage.manage import \
            module, PtahModule, PtahManageRoute, set_access_manager

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        def accessManager(id, request):
            return True

        self.init_ptah()

        set_access_manager(accessManager)
        ptah.auth_service.set_userid('test-user')

        cfg = ptah.get_settings(ptah.CFG_ID_PTAH, self.registry)
        cfg['disable_modules'] = ('test-module',)

        request = DummyRequest()
        route = PtahManageRoute(request)

        self.assertRaises(KeyError, route.__getitem__, 'test-module')

    def test_manage_layout(self):
        from ptah.manage.manage import \
            module, PtahModule, LayoutManage

        @module('test-module')
        class TestModule(PtahModule):
            """ module description """

            title = 'Test module'

        class Content(object):
            __parent__ = None

        self.init_ptah()

        mod = TestModule(None, self.request)
        content = Content()
        content.__parent__ = mod
        self.request.context = content

        layout = LayoutManage(mod, self.request)
        layout.viewcontext = content
        layout.context.userid = ''
        layout.update()

        self.assertIs(layout.module, mod)
